------------------------------------------------------------------------
-- File: setup_msqtschema_posname.sql
-- Author: Joffrey Fitz
-- Usage:
--   psql -U postgres msqtpublic < ./setup_msqtschema_posname.sql
--
-- Clean up with:
-- echo "drop schema yourdatasetname cascade" | psql -U postgres msqtpublic
-- echo "delete from msqtpublic.public.msqt_config where msqt_schema_name='YourDatasetName'" | psql -U postgres msqtpublic
------------------------------------------------------------------------

BEGIN;
------------------------------------------------------------------------
-- Create new schema
------------------------------------------------------------------------
create schema YourDatasetName;

------------------------------------------------------------------------
-- Table structure for a schema of type struct_position_name
--
-- We need 3 tables: 
-- 	ecotypes
--	snps
--	target_sequences
------------------------------------------------------------------------

-- This is an example for YourDatasetName
CREATE TABLE YourDatasetName.ecotypes
(
  ecotype varchar(8) NOT NULL,
  CONSTRAINT YourDatasetName_ecotypes_pkey PRIMARY KEY (ecotype)
);

CREATE TABLE YourDatasetName.snps
(
  position_name varchar(256) NOT NULL,
  ecotype varchar(8) NOT NULL DEFAULT ''::character varying,
  replacement char(1),
  frag_id varchar(256) NOT NULL,
  allele_freq float4 DEFAULT 1,
  CONSTRAINT YourDatasetName_snps_pkey PRIMARY KEY (position_name, ecotype)
);

CREATE TABLE YourDatasetName.target_sequences
(
  position_name varchar(256) NOT NULL,
  ref_base char(1),
  frag_id varchar(256) NOT NULL,
  CONSTRAINT YourDatasetName_target_sequences_pkey PRIMARY KEY (position_name)
);

CREATE TABLE YourDatasetName.sniped
(
  fragid varchar(255) NOT NULL,
  position_name varchar(255) NOT NULL,
  summary text,
  allele1 varchar(1) NOT NULL,
  allele_freq1 float4 NOT NULL,
  allele2 varchar(1) NOT NULL,
  allele_freq2 float4 NOT NULL,
  freq_sum float4 NOT NULL,
  freq_diff float4 NOT NULL,
  neighborhood_left_len int4 NOT NULL,
  neighborhood_right_len int4 NOT NULL,
  ecot1 text NOT NULL,
  ecot2 text NOT NULL,
  adf text NOT NULL
);
------------------------------------------------------------------------
-- Set user privileges for msqtwebuser
------------------------------------------------------------------------
grant usage on SCHEMA YourDatasetName to msqtwebuser;
grant select on TABLE YourDatasetName.snps to msqtwebuser;
grant select on TABLE YourDatasetName.ecotypes to msqtwebuser;
grant select on TABLE YourDatasetName.target_sequences to msqtwebuser;
grant select on TABLE YourDatasetName.sniped to msqtwebuser;

------------------------------------------------------------------------
-- Register type of schema in msqt_config
--
-- Required information: 
--   name of the schema:    arbitrary text
--   type of the schema:    struct_position_name 
--                          or struct_position_chromosome
--   number of chromosomes: only required if type is 
--                          struct_position_chromosome
------------------------------------------------------------------------

-- Example for type struct_position_name (e.g. YourDatasetName alpina data)
INSERT INTO public.msqt_config 
(
	msqt_schema_name,
	msqt_schema_type
)
VALUES (
	'YourDatasetName',
	'struct_position_name'
);


COMMIT;