#!/bin/sh
#
# msqt-setup.sh  -  Setup tool for MSQT
#
# written 2006 by Joffrey Fitz fitzj@informatik.uni-tuebingen.de
#
# This script is part of the MSQT package written by
# Norman Warthmann and Joffrey Fitz
# http://www.eb.tuebingen.mpg.de/msqt
#
#
# Configuration Part:
#
# Adjust these variables according your requirements
#
# WEBROOT	the path to the webroot directory of your webserver
#
# 			Common webroot paths are
# 			/var/www/htdocs            (Apache/Debian Linux)
# 			/var/www/localhost/htdocs  (Apache/Gentoo Linux)
#
# CGIROOT	the path to your cgiroot directory of your webserver
#			
#			Common cgiroot paths are
# 			/usr/lib/cgi-bin           (Apache/Debian Linux)
# 			/var/www/localhost/cgi-bin (Apache/Gentoo Linux)
#
# MSQTWEB   this path will be added to your webroot and locates all
#           needed html files
#
# MSQTCGI   this path will be added to your cgiroot and locates all 
#           MSQT cgis
#
#
# Standard locations of tthe WEBROOT and CGIROOT directories depend on the choice of 
# the webserver and your operatiog system you are running. In doubt ask your
# system administrator.
#
# Setting MSQTWEB to 'msqt' and MSQTCGI to 'msqt' is a good choice.
#
#
# If you are unsure about this configuration issue, please consult your system 
# administrator.


#WEBROOT="/var/www/localhost/htdocs"
#CGIROOT="/var/www/localhost/"

#MSQTWEB="msqt"
#MSQTCGI="cgi-bin/msqt"

#HOST="localhost"
#CGIHOST=$HOST

# Example configuration for merkur
# WEBROOT="/var/www/"
# CGIROOT="/var/www/"
# 
# MSQTWEB="MSQTv.5"
# MSQTCGI="MSQTv.5/cgi-bin"
# 
# HOST="10.39.6.69"
# CGIHOST=$HOST

# Example configuration for merkur (public access)
# WEBROOT="/var/www/"
# CGIROOT="/var/www/"
# 
# MSQTWEB="msqt"
# MSQTCGI="msqt/cgi-bin"
# 
# HOST="www.eb.tuebingen.mpg.de"
# CGIHOST=$HOST

# Example configuration for abt6cs1
#WEBROOT="/var/www/"
#CGIROOT="/usr/lib/cgi-bin"

#MSQTWEB="msqt_intern"
#MSQTCGI="msqt_intern"

#HOST="abt6cs1.eb.local"
#CGIHOST="abt6cs1.eb.local/cgi-bin"


# Example configuration for msqt.weigelworld.org
#WEBROOT="/var/www/msqt"
#CGIROOT="/usr/lib/cgi-bin/msqt"

#MSQTWEB=""
#MSQTCGI=""

#HOST="msqt.weigelworld.org"
#CGIHOST="msqt.weigelworld.org/cgi-bin"


# Database parameters for abt6dbs
#DBNAME="msqt";
#DBHOST="10.39.6.60";
#DBPORT="5432";
#DBPASS="flash2";


# End of configuration.



# DO NOT EDIT ANYTHING BELOW THIS LINE
####################################################################################
CONFIGFILE="../config/msqt.conf"

WORK="./webapp_work"
CGIFILES=`ls ../src/cgi-bin`
HTDOCFILES="../src/msqt_layout_files ../src/html/*"


#######################################################################################
# Function definitions

read_config_file() {
	unset DBHOST DBPORT DBNAME DBPASS WEBROOT CGIROOT MSQTWEB MSQTCGI HOST CGIHOST MSQTADMINUSER MSQTADMINPASS
	if [ -r ${CONFIGFILE} ]; then
		. ${CONFIGFILE}
	else
		fail "Missing or unreadable ${CONFIGFILE}"
	fi
	
	if [ -z "$DBHOST" ] || [ -z "$DBPORT" ] || [ -z "$DBNAME" ] || [ -z "$MSQTWEBROOT" ] || [ -z "$MSQTCGIROOT" ] || [ -z "$STATIC_URL_PREFIX" ] || [ -z "$CGI_URL_PREFIX" ] || [ -z "$MSQTADMINUSER" ] || [ -z "$MSQTADMINPASS" ]; then
		echo "!! Missing configuration DBHOST DBPORT DBNAME WEBROOT CGIROOT MSQTWEB MSQTCGI HOST CGIHOST MSQTADMINUSER and/or  MSQTADMINPASS information"
		echo "!! Check your config file (${CONFIGFILE})."
		exit 1;
	fi
}
#######################################################################################

echo "* Reading configuration"
read_config_file


# BUG! If MSQTCGI is empty, there will be a double slash (//) in the URL.
MSQTURLPREFIX=$STATIC_URL_PREFIX
MSQTCGIPREFIX=$CGI_URL_PREFIX


echo "DBHOST:$DBHOST"
echo "DBPORT:$DBPORT"
echo "DBNAME:$DBNAME"
echo "MSQTWEBROOT:$MSQTWEBROOT"
echo "MSQTCGIROOT:$MSQTCGIROOT"
echo "MSQTURLPREFIX:$MSQTURLPREFIX"
echo "MSQTCGIPREFIX:$MSQTCGIPREFIX"
echo

echo "* Checking for existance of $MSQTWEBROOT and $MSQTCGIROOT"
if [ ! -e $MSQTWEBROOT ]
then
    echo "* Directory $MSQTWEBROOT does not exist. Creating it."
    mkdir -p $MSQTWEBROOT
else
    echo "! $MSQTWEBROOT already exists"
fi

if [ ! -e $MSQTCGIROOT ]
then
    echo "* Directory $MSQTCGIROOT does not exist. Creating it"
    mkdir -p $MSQTCGIROOT
else
    echo "! $MSQTCGIROOT already exists"
fi


echo "* Installing MSQT into $MSQTWEBROOT and $MSQTCGIROOT";

echo "* Creating working directory ($WORK)"
if [ -e $WORK ] 
then
    echo "! Directory $WORK exists. Removing $WORK."
    rm -rf $WORK
fi
mkdir $WORK

echo "* Copying necessary files into working directory"
cp -r $HTDOCFILES $WORK;
for i in $CGIFILES
do
    cp -r "../src/cgi-bin/$i" $WORK
done

echo "* Adjusting MSQTWEBROOT and MSQTCGIROOT";

find $WORK -type f -exec perl -p -i -e "s#\[msqtwebroot\]#$MSQTURLPREFIX#g" {} \;
find $WORK -type f -exec perl -p -i -e "s#\[msqtcgiroot\]#$MSQTCGIPREFIX#g" {} \;
find $WORK -type f -exec perl -p -i -e "s#\[webrootpath\]#$MSQTWEBROOT#g" {} \; # used by msqtadmin.cgi

echo "* Adjusting Database parameters";

find $WORK -type f -exec perl -p -i -e "s#\[DBNAME\]#$DBNAME#g" {} \;
find $WORK -type f -exec perl -p -i -e "s#\[DBHOST\]#$DBHOST#g" {} \;
find $WORK -type f -exec perl -p -i -e "s#\[DBPORT\]#$DBPORT#g" {} \;
find $WORK -type f -exec perl -p -i -e "s#\[DBPASS\]#$DBPASS#g" {} \;

find $WORK -type f -exec perl -p -i -e "s#\[MSQTADMINUSER\]#$MSQTADMINUSER#g" {} \;
find $WORK -type f -exec perl -p -i -e "s#\[MSQTADMINPASS\]#$MSQTADMINPASS#g" {} \;

echo "* Installing MSQT"
for i in $HTDOCFILES 
do
	FILENAME=`basename $i`
    cp -rv "$WORK/$FILENAME" $MSQTWEBROOT
    chgrp -R $WEBSERVERUSER $MSQTWEBROOT
    chown -R $WEBSERVERGROUP $MSQTWEBROOT
done

for i in $CGIFILES 
do
    cp -rv "$WORK/$i" $MSQTCGIROOT
    chgrp -R $WEBSERVERUSER $MSQTCGIROOT
    chown -R $WEBSERVERGROUP $MSQTCGIROOT
done

echo "* Cleaning up"
rm -rf $work

echo 
#echo "Later your fasta files will have to go into $MSQTWEBROOT/fasta_files/"
echo 
echo "Installation finished."
echo "Point your browser to $MSQTURLPREFIX/index.html"
echo "May MSQT be with you." 
