#!/usr/bin/perl
use strict;

#
# alivie0.4.cgi
#
# (c) 2006 by Joffrey Fitz <fitzj@informatik.uni-tuebingen.de>
#
# This program is free software; you can redistribute it and/or modify it                        
# under the terms of the GNU General Public License as published by the Free                     
# Software Foundation; either version 2, or (at your option) any later 
# version.
#
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
# or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
# for more details.
#

use LWP::Simple;
use CGI;
use CGI::Carp qw( fatalsToBrowser );
use File::Basename;
use URI;

my $VERSION="0.4";
my $DEBUG = 1;
$| = 1;

#
# Config: default values
#
my $file_type = "fasta";
my $ruler = 1;
# ruler offset
my $offset=10;
my @species; # if empty all species in fasta files are shown
#my $uri = $ARGV[0];

#
# parse CGI params
#
my $q = new CGI;

print "Content-type: text/html\n\n";

my $uri;
if( defined $q->param( "file" ) ) {
	$uri = $q->param( "file" );
}
die "No input.\n" unless $uri;

if( defined $q->param( "offset" ) ) {
	$offset = $q->param( "offset" );
}

if( defined $q->param( "species" ) ) {
	@species = split(/ /,$q->param( "species" ));
}


my ($url, $file, $fasta_file_name, @content);
if($uri =~ /^http:\/\//) {
	# Thats a web URL
	my $url = URI->new($uri);
	$fasta_file_name =  basename( $url->path() );
#$url = $uri;
	my $buf = get $url;
  	die "Couldn't get $url" unless defined $buf;
  	@content = split(/\n/, $buf);
}
else {
	$file = $uri;
	$fasta_file_name = basename($file);
	open(FH,$file) or die "$!: $file\n";
	@content = <FH>;
}



print <<EOH;
<html>
<head>
    <title>Alivie</title>
    <style type="text/css">
	<!--
	h1 { font-family:Verdana,sans-serif; font-weight:bold;}
	.A {  background-color:#FF7D7F;}
	.G {  background-color:#3E58FF;}
	.T {  background-color:#45BC70;}
	.C {  background-color:#AC31BC;}
	.N {  background-color:#000000; color:#AAAAAA; }
	.gap {  background-color:#FFFFFF; }

	#ruler { background-color:#D2D3D9 }

    #container {
		position:absolut; 
		top5px; left:5px; width:775px; height:350px; 
		z-index:0; padding:5;
	 	overflow:scroll; 
		background: #EFEFEF; border:thin solid #804000
	}

	div.table { 
		display:table; border-collapse:collapse;
		font-family:'Courier New',Courier,Monospace,fixed;
		font-size:12px;
		line-height:12px }
	div.tr { display:table-row; padding:0px }
	div.td { display:table-cell; padding:0px; 
		 word-wrap: normal; 
	}
	-->
	</style>
</head>
<body bgcolor="FFFFFF" text="#000000">

<!--<h1>$fasta_file_name</h1>-->

<div id="container">
<div class="table">
EOH

my $buf;
my $len=0;
while(@content) {
	my $line = shift @content;
	chomp $line;
	if($line =~ />(.*)$/) {
		$buf .= "  <div class=\"tr\">\n    <div class=\"td\"><b>$1</b></div>\n";
	}
	else {
		$len = length($line) if ($len==0);
		
		$line =~ s/(A+)/<span class="A">$1<\/span>/g;
		$line =~ s/(G+)/<span class="G\">$1<\/span>/g;
		$line =~ s/(T+)/<span class="T\">$1<\/span>/g;
		$line =~ s/(C+)/<span class="C\">$1<\/span>/g;
		$line =~ s/(N+)/<span class="N\">$1<\/span>/g;
		$line =~ s/(-)/<span class="gap\">$1<\/span>/g; # do not cluster, bug in Safari
		$buf .= "   <div class=\"td\">$line&nbsp</div>\n  </div>\n";
	}
}

# this is murks
# need time to rewrite ruler logic
my $unit=0;
my $digits=1;
my $ruler;
print STDERR "len:$len\n";
for(my $i=1;$i<$len/$offset;$i++) {
	if($unit==0) { ;}
	elsif($unit+$offset==100) { $digits = $digits+1;}
	elsif($unit+$offset==1000) { $digits = $digits+2;}
	
	print STDERR "i:$i digits:$digits unit:$unit offset:$offset\n";
	for(my $j=1;$j<$offset-$digits;$j++) {
		print STDERR "j:$j\n";
		$ruler .= "&nbsp;";
	}
	$unit+=$offset;
	$ruler .= $unit;
}
my $subruler;
for(my $i=1;$i<$len/$offset;$i++) {
	for(my $j=1;$j<$offset;$j++) {
		$subruler .= ".";
	}
	$subruler .= "|";
}

print STDERR "ruler:$ruler\nsubruler:$subruler\n";


print <<EOF;
  <div class=\"tr\">\n    <div class=\"td\"></div>
  <div class=\"td\" id="ruler">$ruler</div>\n  </div>
  <div class=\"tr\">\n    <div class=\"td\"></div>
  <div class=\"td\" id="ruler">$subruler</div>\n  </div>
$buf
</div>

</div>

<hr>
Alivie v$VERSION &copy; 2006 by Joffrey Fitz (fitzj\@informatik.uni-tuebingen.de)
</body></html>
EOF

close FH;
exit 0;
