#!/usr/bin/perl -w

#
# msqt-adf.cgi
#
# Author: Joffrey Fitz <fitzj@informatik.uni-tuebingen.de>
#
# Copyright (c) 2007 Max Planck Institute for Developmental Biology, 
# 	http://www.eb.tuebingen.mpg.de
#
# This program is part of the MSQT project by Norman Warthmann, Joffrey Fitz 
# and Detlef Weigel (http://msqt.weigelworld.org)
#
# This program is free software; you can redistribute it and/or modify it                        
# under the terms of the GNU General Public License as published by the Free                     
# Software Foundation; either version 2, or (at your option) any later 
# version.
#
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
# or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
# for more details.
#

use strict;
use CGI;
use CGI::Carp qw( fatalsToBrowser );
use DBI;
use Switch;	# required by Norman
use XML::Simple;
use distinct_snps_chrpos;
use distinct_snps_posname;

# global
my $DEBUG = 0;

# CGI
my $script_loc = "[msqtcgiroot]/msqt-adf.cgi";

# database localhost
#my $dbname = "msqt";
#my $host = "127.0.0.1";
#my $port = "5432";
#my $options = "";
#my $username = "postgres";
#my $password = "";

# database config
my $dbname = "[DBNAME]";
my $host = "[DBHOST]";
my $port = "[DBPORT]";
my $options = "";
my $username = "msqt";
my $password = "[DBPASS]";


my $dbh;
my $sth;
my $row;
my $query;

my $schema_type;

# GUI templates
# my $header_file = "html/msqt-header.html";
# my $content_file_chrpos = "html/msqt-adf-content-chrpos.html"; # for chromosome/position data
# my $content_file_posname = "html/msqt-adf-content-posname.html";# for position-name data
# my $footer_file = "html/msqt-footer.html";
my $error_file = "html/server_down_error.html";
my $content_file_chrpos = "html/tmpl_msqt-adf-chrpos.html";
my $content_file_posname = "html/tmpl_msqt-adf-posname.html";

# vars for the widgets
my $dbtable;
my $position;

my $chromosome;
my $database;
my $ecot_list1;		# store the selected ecotypes for list 1
my $ecot_list2;		# and list 2
my $ecot_list3;

my $frag_id;            # for alpina, guppy
my $pos_name;           # for alpina, guppy


# buffer for the html-results
my $html_results;


####################################################################


# do not buffer
$| = 1;

print "Content-type: text/html\n\n";

$dbh = DBI->connect("dbi:Pg:dbname=$dbname;host=$host;port=$port;".
	"", "$username", "$password",
        {AutoCommit => 1} ) || db_connect_error();



&parse_params();
&print_content();


####################################################################


$dbh->disconnect || die;
exit 0;


####################################################################

sub print_content {
	my $file_buf;
	my $combobox_buf;
	my $listbox_buf1;
	my $listbox_buf2;
	my $listbox_buf3;
	my $chromosome_buf;

    my $combobox_options;
	my $listbox_options;
	my $chromosome_options;

	# ecotype table: user submitted table with suffix _ecotypes

	# If $dbtable is not defined, get default dataset from db
	if($dbtable eq "") {
		my $query = "SELECT msqt_schema_name FROM public.msqt_config WHERE default_dataset = true";
		my $sth = $dbh->prepare( $query );
		$sth->execute() || print "<font color=red><b>" . $sth->errstr . "</b></font><br><br>";
		while ( $row = $sth->fetchrow ) {
			$dbtable = $row;
		}
		$sth->finish;
		
		if($DEBUG) { print "got default dataset from db: $dbtable<br>"; }
	}
	
	# Determine which database structure is used
	$query = "SELECT msqt_schema_type FROM public.msqt_config WHERE msqt_schema_name = '$dbtable'";
	if($DEBUG) { print "query: $query<br>"; }

	$sth = $dbh->prepare( $query );
	$sth->execute() || print "<font color=red><b>" . $sth->errstr . "</b></font><br><br>";
	while ( $row = $sth->fetchrow ) {
	$schema_type = $row;
	}
	if($DEBUG) { print "schema-type: $schema_type<br>"; }
	if($schema_type eq "struct_position_name") {
	#if($DEBUG) { print "struct_position_name:"; }
	#if($DEBUG) { print "  reading template '$content_file_posname' for schema-type: $schema_type<br>"; }
	open FORM, $content_file_posname or die $!;
	# read template
	while(<FORM>) {
		$file_buf .= $_;
	}
	close FORM;
	}
	elsif($schema_type eq "struct_position_chromosome") {
	#if($DEBUG) { print "*reading template '$content_file_chrpos' for schema-type: $schema_type<br>"; }
	open FORM, $content_file_chrpos or die $!;
	# read template
	while(<FORM>) {
		$file_buf .= $_;
	}
		close FORM;
	}
	else {
		die ("configuration error: no template found for $dbtable ($row).")
	}


	# Insert headline
	$file_buf =~ s/--headline--/MSQT\/<b>ADF<\/b>/;
	
	# Insert scriptloc into FORM tag
	$file_buf =~ s/--MSQT-TEMPL:scriptloc--/$script_loc/;
	

	# Generate tabbar
	my $tabbar_buf;
	$tabbar_buf = <<HTML;
	<div class="tabbar_table">
			<div class="tabbar_tr">
				<span class="tabbar_td"><a class="tabbar_link" href="[msqtwebroot]/index.html">Home</a></span> <span class="toolbar_divider">&nbsp;&nbsp;&nbsp;&nbsp;</span>
				<span class="tabbar_td"><a class="tabbar_link" href="[msqtcgiroot]/msqt-sbe.cgi">SNPs Between Ecotypes</a></span> <span class="toolbar_divider">&nbsp;&nbsp;&nbsp;&nbsp;</span>
				<span class="tabbar_td"><a class="tabbar_link" href="[msqtcgiroot]/msqt-adf.cgi">Assay Development Formater</a></span> <span class="toolbar_divider">&nbsp;&nbsp;&nbsp;&nbsp;</span>
				<span class="tabbar_td"><a class="tabbar_link" href="[msqtcgiroot]/msqt-e.cgi">Expert Mode</a></span> <span class="toolbar_divider">&nbsp;&nbsp;&nbsp;&nbsp;</span>
				<span class="tabbar_td"><a class="tabbar_link" href="[msqtcgiroot]/msqt-sniped.cgi">SNIPED!</a></span> <span class="toolbar_divider">&nbsp;&nbsp;&nbsp;&nbsp;</span>
				<span class="tabbar_td"><a class="tabbar_link" href="[msqtcgiroot]/msqtadmin/msqtadmin.cgi" target="__msqtadmin__">Admin</a></span> <span class="toolbar_divider">&nbsp;&nbsp;&nbsp;&nbsp;</span>
				<span class="tabbar_td"><a class="tabbar_link" href="[msqtwebroot]/doc.html">Documentation</a></span> <span class="toolbar_divider">&nbsp;&nbsp;&nbsp;&nbsp;</span>
				<span class="tabbar_td"><a class="tabbar_link" href="[msqtwebroot]/about.html">About</a></span>
			</div>
		</div>
HTML
	$file_buf =~ s/<!--MSQT-TEMPL:tabbar-->/$tabbar_buf/;
	
	# generate schema combobox
	# query existing schemas from database
	$query = "SELECT msqt_schema_name FROM public.msqt_config";
	if($DEBUG) { print "query: $query<br>"; }
	$sth = $dbh->prepare( $query );
	$sth->execute() || print "<font color=red><b>" . $sth->errstr . "</b></font><br><br>";

    # generate options for combobox
	my $row_cnter=0;
	my @row_arr;
	while ( @row_arr = $sth->fetchrow_array ) {
		$combobox_options .= "<option value=\"" . $row_arr[0] . "\"";
		$combobox_options .= ">$row_arr[0]</option>\n";
		$row_cnter++;
	}
	$sth->finish();

	# generate combobox
	$combobox_buf = "<select name=\"sbe_database\" tabindex=\"1\">";
	$combobox_buf .= $combobox_options;
	$combobox_buf .= "</select>";

	# select right entry
	if($DEBUG) { print "dbtable: $dbtable<br>"; }
	$combobox_buf =~ s/value="$dbtable"/value="$dbtable" selected/;
	
	# insert combobox into template
	$file_buf =~ s/<!--schema_list-->/$combobox_buf/;


	# generate ecotypes listboxes
	# query ecotype lists from database
	$query = "SELECT * FROM $dbtable.ecotypes WHERE ecotype !='target'";

	if($DEBUG) { print "query: $query<br>"; }

	$sth = $dbh->prepare( $query );
	$sth->execute() || print "<font color=red><b>" . $sth->errstr . "</b></font><br><br>";

	# generate options for listboxes
	my $row_cnter=0;
	while ( $row = $sth->fetchrow ) {
		$listbox_options .= "<option value=\"" . $row .
			"\">$row</option>\n";
		$row_cnter++;
	}
	$sth->finish();

	# generate listboxes

	$listbox_buf1 = "<select name=\"ecot_list_1\" size=\"15\" multiple=\"multiple\">";
	$listbox_buf1 .= $listbox_options;
	$listbox_buf1 .= "</select>";


	$listbox_buf2 = "<select name=\"ecot_list_2\" size=\"15\" multiple=\"multiple\">";
	$listbox_buf2 .= $listbox_options;
	$listbox_buf2 .= "</select>";

	$listbox_buf3 = "<select name=\"ecot_list_3\" size=\"15\" multiple=\"multiple\">";
	$listbox_buf3 .= $listbox_options;
	$listbox_buf3 .= "</select>";


        # select the right ecotypes in listbox
        foreach my $entry (split(/,/, $ecot_list1)) {
          $entry =~ s/'//g;
          if( !grep( /^$entry$/, $listbox_buf1) ) {
            $listbox_buf1 =~ s/value="$entry"/value="$entry" selected/;
          }
        }
        foreach my $entry (split(/,/, $ecot_list2)) {
          $entry =~ s/'//g;
          if( !grep( /^$entry$/, $listbox_buf2) ) {
            $listbox_buf2 =~ s/value="$entry"/value="$entry" selected/;
          }
        }
        foreach my $entry (split(/,/, $ecot_list3)) {
          $entry =~ s/'//g;
          if( !grep( /^$entry$/, $listbox_buf3) ) {
            $listbox_buf3 =~ s/value="$entry"/value="$entry" selected/;
          }
        }


	# insert listboxes into template
	$file_buf =~ s/<!--ecotype_list_1-->/$listbox_buf1/;
	$file_buf =~ s/<!--ecotype_list_2-->/$listbox_buf2/;
	$file_buf =~ s/<!--ecotype_list_3-->/$listbox_buf3/;

	# insert results into template
	$file_buf =~ s/<!--results-->/$html_results/;


    # fill out other fields
  	if($schema_type eq "struct_position_name") {
		# insert frag_id and pos_name
		if($frag_id) {
        	$file_buf =~ s/--frag_id--/$frag_id/;
        }
        else {
        	$file_buf =~ s/--frag_id--//;
        }
		if($pos_name) {
        	$file_buf =~ s/--pos_name--/$pos_name/;
        }
        else {
         	$file_buf =~ s/--pos_name--//;
        }
  	}
    elsif($schema_type eq "struct_position_chromosome") {
	
            if($position) {
              $file_buf =~ s/--pos_value--/$position/;
            }
            else {
				$file_buf =~ s/--pos_value--//;
            }

#           # end position
#           if($end_pos) {
#             $file_buf =~ s/--end_pos_value--/$position/;
#           }
#           else {
#             $file_buf =~ s/--end_pos_value--/0/;
#           }

          #chromosome

          # generate chromosome list
          # query number of chromosomes from database
          $query = "SELECT chromosomes FROM public.msqt_config WHERE msqt_schema_name='$dbtable'";
          if($DEBUG) { print "query: $query<br>"; }
          $sth = $dbh->prepare( $query );
          $sth->execute() || print "<font color=red><b>" . $sth->errstr . "</b></font><br><br>";
          my @row_chromosomes = $sth->fetchrow_array;
          $sth->finish();

          # generate options for chromosome list
          for(my $i=1; $i<=$row_chromosomes[0];$i++) {
                  $chromosome_options .= "<option value=\"" . $i . "\">$i</option>\n";
          }

          # generate chromosome list
          $chromosome_buf = "<select name=\"chromosome\" tabindex=\"3\">";
          $chromosome_buf .= $chromosome_options;
          $chromosome_buf .= "</select>";

          # select right entry
          if($dbtable) {
            $chromosome_buf =~ s/value="$chromosome"/value="$chromosome" selected/;
          }

          # insert chromosome list into template
          $file_buf =~ s/<!--chromosome_list-->/$chromosome_buf/;

        }


	print $file_buf;
	close FORM;
}

# sub print_footer {
# 	open FOOTER, $footer_file or die $!;
# 	print <FOOTER>;
# 	close FOOTER;
# }

sub parse_params {
	my $q = new CGI;
	my( $name, $wert );
	my $buf = "";		# buffer for db results
	my $header_str;		# result header


	# get the selected ecotypes for list 1
	foreach $name ( $q->param ) {
		if($name =~ /ecot_list_1/) {
			if($DEBUG) { print "found $name<br>"; }
			foreach $wert ( $q->param( $name ) ) {
				$ecot_list1 .= "'$wert',";
			}
		}
	}
	chop($ecot_list1); # remove last ","

	# get the selected ecotypes for list 2
	foreach $name ( $q->param ) {
		if($name =~ /ecot_list_2/) {
			if($DEBUG) { print "found $name<br>"; }
			foreach $wert ( $q->param( $name ) ) {
				$ecot_list2 .= "'$wert',";
			}
		}
	}
	chop($ecot_list2); # remove last ","

	# get the selected ecotypes for list 2
	foreach $name ( $q->param ) {
		if($name =~ /ecot_list_3/) {
			if($DEBUG) { print "found $name<br>"; }
			foreach $wert ( $q->param( $name ) ) {
				$ecot_list3 .= "'$wert',";
			}
		}
	}
	$ecot_list3 .= "'target',"; # add target
	chop($ecot_list3); # remove last ","

	# check action and if($DEBUG) { print "query: $query"; }perform db qureies
	if( defined $q->param( "change_db" ) ) {
		$dbtable 	= $q->param( "sbe_database" );
		$html_results = "Database changed to '$dbtable'.";

	}
	if( defined $q->param( "compute_snps" ) ) {

	        $dbtable = $q->param( "sbe_database" );
                $query = "SELECT msqt_schema_type FROM public.msqt_config WHERE msqt_schema_name = '$dbtable'";
                if($DEBUG) { print "compute_snps: query: $query<br>"; }

                $sth = $dbh->prepare( $query );
                $sth->execute() || print "<font color=red><b>" . $sth->errstr . "</b></font><br><br>";
                while ( $row = $sth->fetchrow ) {
                  $schema_type = $row;
                }

                if($schema_type eq "struct_position_name") {
                  $frag_id = $q->param( "frag_id" );
                  $pos_name = $q->param( "pos_name" );
                }
                elsif($schema_type eq "struct_position_chromosome") {
                  $position 	= $q->param( "position" );
		  		  $chromosome = $q->param( "chromosome" );
                }
                else {
                  die "schema type undefined for database '$dbtable'";
                }

                if($DEBUG) { print "<b>Parameter found: frag_id=$frag_id, pos_name=$pos_name, ...</b><br>";}

                # Do computation
                if($schema_type eq "struct_position_name") {
                
					# if no frag_id is given, we must get it from the database
					if( ($frag_id eq "") and ($pos_name ne "") ) {
						my $q_frag = "SELECT distinct frag_id FROM $dbtable.snps where position_name='$pos_name';";
					
						my $sth_frag = $dbh->prepare($q_frag);
						$sth_frag->execute();
						my $ref_frag = $sth_frag->fetchrow_hashref();
						$frag_id = $ref_frag->{'frag_id'};
							
						print "<pre><b>Debug: distinct_snps_posname:find_frag</b>\n";
						print "frag_id missing -> querying frag_id\nQuery:$q_frag\nres:$frag_id</pre>";
					}
					
                  	gen_genaissance_format_posname();
                }
                elsif($schema_type eq "struct_position_chromosome") {
                  gen_genaissance_format_chrpos();
                }
	}
}


# called by parse_params()
sub gen_genaissance_format_posname {

	# Parameters submitted by GUI
	# string containing ecotypes must be converted into hash
	my %ethash;
	$ecot_list1 =~ s/'//g;
	$ecot_list2 =~ s/'//g;
	$ecot_list3 =~ s/'//g;

	foreach my $et (split(",", $ecot_list1)) {
		$ethash{$et} = 0;
	}
	foreach my $et (split(",", $ecot_list2)) {
		$ethash{$et} = 1;
	}
	foreach my $et (split(",", $ecot_list3)) {
		$ethash{$et} = 2;
	}


	$html_results  = "Query initiated with following parameters:<br><br>";
	#$html_results .= "chromosome = $chromosome<br>";
	$html_results .= "pos_name  = $pos_name<br>";
	$html_results .= "ecot_list1 = $ecot_list1<br>";
	$html_results .= "ecot_list2 = $ecot_list2<br>";
        $html_results .= "also annotated: $ecot_list3<br><br><p>";

	if($DEBUG) { print "gen_genaissance_format_posname: dbtable: $dbtable  "; }

	my $snp_list = new distinct_snps_posname();

	# Initialize snp_list for a special position (and its fragment)
	# and generate genaissance format
	
	if($DEBUG) {
		print "<pre>msqt-adf.cgi:gen_genaissance_format_posname:\n",
			"frag_id:$frag_id, pos_name:$pos_name</pre>";
	}
	
	# If pos_name is given (3rd parameter) only primary SNP will be returned
    $snp_list->find_frag(1, $frag_id, "", $dbh, $dbtable, %ethash);
	# generate xml
	$snp_list->genaissance_xml_experimental($dbh,$pos_name);
	
	# get xml output
	my $output = $snp_list->{gen_out};
	if($output ne "") {
		# parse xml
		my $xs = new XML::Simple();
		my $adf_xml = $xs->XMLin($output, forcearray => ['fragment', 'ecotype', 'secondary_snp']);
		
		if($adf_xml->{structure} eq "posname") {
			foreach my $fragment (@{$adf_xml->{fragment}}) {
				$fragment->{adf} =~ s/(\[.+?\])/<font color="red"><b>$1<\/b><\/font>/g;
				$fragment->{adf} =~ s/(\{.+?\})/<font color="blue"><b>$1<\/b><\/font>/g;
				
				# Print the ADF string
				$html_results .= $fragment->{position_name} . ":" . $fragment->{adf} . "\n";
			}
		}
	}
	else {
		$html_results .= "No results";
	}
	
	$html_results .= "<br><p>ready.";
}

sub gen_genaissance_format_chrpos {

	# Parameters submitted by GUI
	# string containing ecotypes must be converted into hash
	my %ethash;
	$ecot_list1 =~ s/'//g;
	$ecot_list2 =~ s/'//g;
	$ecot_list3 =~ s/'//g;

	foreach my $et (split(",", $ecot_list1)) {
		$ethash{$et} = 0;
	}
	foreach my $et (split(",", $ecot_list2)) {
		$ethash{$et} = 1;
	}
	foreach my $et (split(",", $ecot_list3)) {
		$ethash{$et} = 2;
	}


	$html_results  = "Query initiated with following parameters:<br><br>";
	$html_results .= "chromosome = $chromosome<br>";
	$html_results .= "position  = $position<br>";
	$html_results .= "ecot_list1 = $ecot_list1<br>";
	$html_results .= "ecot_list2 = $ecot_list2<br>";
    $html_results .= "also annotated: $ecot_list3<br><br>";

	if($DEBUG) { print "dbtable: $dbtable  "; }

	my $snp_list = new distinct_snps_chrpos();

	# Initialize snp_list for a special position (and its fragment)
	# and generate genaissance format
	$snp_list->find_frag($chromosome, $position, $dbh, $dbtable, %ethash);
	# generate xml
	$snp_list->genaissance_xml_experimental($dbh, $position);

	my $output = $snp_list->{gen_out};
    
    if($output ne "") {
		my $xs = new XML::Simple();
		my $adf_xml = $xs->XMLin($output, forcearray => ['fragment', 'ecotype', 'secondary_snp']);
	
		if($adf_xml->{structure} eq "chrpos") {
			foreach my $fragment (@{$adf_xml->{fragment}}) {
					# Highlight SNPs
				$fragment->{adf} =~ s/(\[.+?\])/<font color="red"><b>$1<\/b><\/font>/g;
				$fragment->{adf} =~ s/(\{.+?\})/<font color="blue"><b>$1<\/b><\/font>/g;
				
				# Print the ADF string
				$html_results .= $fragment->{polymorphisms}->{primary_snp}->{position} . ":" . $fragment->{adf} . "\n";
			}
		}
	}
	else {
		$html_results .= "No results";
	}

	$html_results .= "<br><p>ready.";
}

sub db_connect_error {
	open FORM, $error_file or die $!;
	# read template
	while(<FORM>) {
		print $_;
	}
	exit(0);
}

sub min {
    if ($_[0]>$_[1]) {return $_[1]} else {return $_[0]};
}
