#!/usr/bin/perl -w

#
# msqt-e.cgi
#
# Author: Joffrey Fitz <fitzj@informatik.uni-tuebingen.de>
#
# Copyright (c) 2007 Max Planck Institute for Developmental Biology, 
# 	http://www.eb.tuebingen.mpg.de
#
# This program is part of the MSQT project by Norman Warthmann, Joffrey Fitz 
# and Detlef Weigel (http://msqt.weigelworld.org)
#
# This program is free software; you can redistribute it and/or modify it                        
# under the terms of the GNU General Public License as published by the Free                     
# Software Foundation; either version 2, or (at your option) any later 
# version.
#
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
# or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
# for more details.
#

use strict;
use CGI;
use CGI::Carp qw( fatalsToBrowser );
use DBI;
use Switch;	# required by Norman

# global
my $DEBUG = 0;

# CGI
my $script_loc = "[msqtcgiroot]/msqt-e.cgi";

# database localhost
#my $dbname = "msqt";
#my $host = "127.0.0.1";
#my $port = "5432";
#my $options = "";
#my $username = "postgres";
#my $password = "";

# database config
my $dbname = "[DBNAME]";
my $host = "[DBHOST]";
my $port = "[DBPORT]";
my $options = "";
my $username = "msqt";
my $password = "[DBPASS]";


my $dbh;
my $sth;
my $row;
my $query;

# GUI templates
# my $header_file = "html/msqt-header.html";
# my $content_file = "html/msqt-e-content.html";
# my $footer_file = "html/msqt-footer.html";
my $error_file = "html/server_down_error.html";
my $content_file = "html/tmpl_msqt-e.html";

# vars for the widgets
my $default_dbtable = "[DEFAULTDATASET]";
my $usr_sql_query;
my $ecot_list1;		# store the selected ecotypes for list 1
my $ecot_list2;		# and list 2

# buffer for the html-results
my $html_results;


####################################################################


# do not buffer
$| = 1;

print "Content-type: text/html\n\n";

$dbh = DBI->connect("dbi:Pg:dbname=$dbname;host=$host;port=$port;".
	"", "$username", "$password",
        {AutoCommit => 0} ) || db_connect_error();



&parse_params();
&print_content();


####################################################################


$dbh->disconnect || die;
exit 0;


####################################################################


sub print_content {
	my $file_buf;
	
	open FORM, $content_file or die $!;
	
	# read template
	while(<FORM>) {
		$file_buf .= $_; 
	}
	
	# Insert headline
	$file_buf =~ s/--headline--/MSQT\/<b>E<\/b>/;
	
	# Insert scriptloc into FORM tag
	$file_buf =~ s/--MSQT-TEMPL:scriptloc--/$script_loc/;
	

	# Generate tabbar
	my $tabbar_buf;
	$tabbar_buf = <<HTML;
	<div class="tabbar_table">
			<div class="tabbar_tr">
				<span class="tabbar_td"><a class="tabbar_link" href="[msqtwebroot]/index.html">Home</a></span> <span class="toolbar_divider">&nbsp;&nbsp;&nbsp;&nbsp;</span>
				<span class="tabbar_td"><a class="tabbar_link" href="[msqtcgiroot]/msqt-sbe.cgi">SNPs Between Ecotypes</a></span> <span class="toolbar_divider">&nbsp;&nbsp;&nbsp;&nbsp;</span>
				<span class="tabbar_td"><a class="tabbar_link" href="[msqtcgiroot]/msqt-adf.cgi">Assay Development Formater</a></span> <span class="toolbar_divider">&nbsp;&nbsp;&nbsp;&nbsp;</span>
				<span class="tabbar_td"><a class="tabbar_link" href="[msqtcgiroot]/msqt-e.cgi">Expert Mode</a></span> <span class="toolbar_divider">&nbsp;&nbsp;&nbsp;&nbsp;</span>
				<span class="tabbar_td"><a class="tabbar_link" href="[msqtcgiroot]/msqt-sniped.cgi">SNIPED!</a></span> <span class="toolbar_divider">&nbsp;&nbsp;&nbsp;&nbsp;</span>
				<span class="tabbar_td"><a class="tabbar_link" href="[msqtcgiroot]/msqtadmin/msqtadmin.cgi" target="__msqtadmin__">Admin</a></span> <span class="toolbar_divider">&nbsp;&nbsp;&nbsp;&nbsp;</span>
				<span class="tabbar_td"><a class="tabbar_link" href="[msqtwebroot]/doc.html">Documentation</a></span> <span class="toolbar_divider">&nbsp;&nbsp;&nbsp;&nbsp;</span>
				<span class="tabbar_td"><a class="tabbar_link" href="[msqtwebroot]/about.html">About</a></span>
			</div>
		</div>
HTML
	$file_buf =~ s/<!--MSQT-TEMPL:tabbar-->/$tabbar_buf/;
	
	# insert sql query in textarea
	$file_buf =~ s/<!--usr_sql_query-->/$usr_sql_query/;
	# insert results into template
	$file_buf =~ s/<!--results-->/$html_results/;

	print $file_buf;
	close FORM;
}


sub parse_params {
	my $q = new CGI;
	my( $name, $wert );
	my $buf = "";		# buffer for db results
	my $header_str;		# result header
	
	# check action and if($DEBUG) { print "query: $query"; }perform db qureies
	if( defined $q->param( "exec_sql_query" ) ) {
		$usr_sql_query = $q->param( "usr_sql_query" );
		execute_sql();
	}
}


# called by parse_params()
sub execute_sql {
	
	if($DEBUG) { print "query: $usr_sql_query<br>"; }
	$sth = $dbh->prepare( $usr_sql_query );# || 
		#$html_results = "<font color=red><b>" . $sth->errstr . "</b></font><br><br>";

	if (not $sth->execute()) {
		$html_results = "<font color=red><b>" . $sth->errstr . "</b></font><br><br>";
		return;
	}
	
	my @col_headers = @{$sth-> {NAME_uc}}; # Now you have the field names in @Header.
	my $col_num = scalar(@col_headers);
	
	$html_results .= "<table border=0 cellspacing=0 >";
	$html_results .= "<tr bgcolor=\"lightgrey\">";
	foreach my $td (@col_headers) {
			$html_results .= "<td class=\"msqttableresult\"><b>$td</b>     </td>";
	}
	$html_results .= "</tr>";
	# generate options for listboxes
	while ( my @row_arr = $sth->fetchrow_array ) {
		$html_results .= "<tr >";
		foreach my $td (@row_arr) {
			$html_results .= "<td class=\"msqttableresult\">$td</td>\n";
		}
		$html_results .= "</tr>";
	}
	$html_results .= "</table>";
	$sth->finish();
	
	$html_results .= "<br><p>ready.";	
}

sub db_connect_error {
	open FORM, $error_file or die $!;
	# read template
	while(<FORM>) {
		print $_; 
	}
	exit(0);
}
