#!/usr/bin/perl -w

#
# msqt-sbe.cgi
#
# Author: Joffrey Fitz <fitzj@informatik.uni-tuebingen.de>
#
# Copyright (c) 2007 Max Planck Institute for Developmental Biology, 
# 	http://www.eb.tuebingen.mpg.de
#
# This program is part of the MSQT project by Norman Warthmann, Joffrey Fitz 
# and Detlef Weigel (http://msqt.weigelworld.org)
#
# This program is free software; you can redistribute it and/or modify it                        
# under the terms of the GNU General Public License as published by the Free                     
# Software Foundation; either version 2, or (at your option) any later 
# version.
#
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
# or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
# for more details.
#

use strict;
use CGI;
use CGI::Carp qw( fatalsToBrowser );
use DBI;
use Switch;	# required by Norman
use distinct_snps_chrpos;
use distinct_snps_posname;
use XML::Simple;

# global
my $DEBUG = 0;

# CGI
my $script_loc = "[msqtcgiroot]/msqt-sbe.cgi";
my $alivie_cgi = "[msqtcgiroot]/msqt-alivie.cgi";
my $adf_cgi = 	 "[msqtcgiroot]/msqt-adf.cgi";

# database localhost
#my $dbname = "msqt";
#my $host = "127.0.0.1";
#my $port = "5432";
#my $options = "";
#my $username = "postgres";
#my $password = "";

# database config
my $dbname = "[DBNAME]";
my $host = "[DBHOST]";
my $port = "[DBPORT]";
my $options = "";
my $username = "msqt";
my $password = "[DBPASS]";


my $dbh;
my $sth;
my $row;
my $query;

# GUI templates
# my $header_file = "html/msqt-header.html";
# my $content_file_chrpos = "html/msqt-sbe-content-chrpos.html"; # for chromosome/position data
# my $content_file_posname = "html/msqt-sbe-content-posname.html";# for position-name data
# my $footer_file = "html/msqt-footer.html";
my $error_file = "html/server_down_error.html";
my $content_file_chrpos = "html/tmpl_msqt-sbe-chrpos.html";
my $content_file_posname = "html/tmpl_msqt-sbe-posname.html";

# vars for the widgets
my $default_dbtable;
my $dbtable;
my $start_pos;
my $end_pos;
my $chromosome;
my $database;
my $ecot_list1;		# store the selected ecotypes for list 1
my $ecot_list2;		# and list 2
my $ecot_list3;		# and list 3
my $frag_id;            # for alpina, guppy
my $pos_name;           # for alpina, guppy
my $schema_type;

# buffer for the html-results
my $html_results;


####################################################################


# do not buffer
$| = 1;

print "Content-type: text/html\n\n";

$dbh = DBI->connect("dbi:Pg:dbname=$dbname;host=$host;port=$port;".
	"", "$username", "$password",
        {AutoCommit => 0} ) || db_connect_error();



&parse_params();
&print_content();

####################################################################


$dbh->disconnect || die;
exit 0;



####################################################################


sub print_content {
	my $file_buf;
	my $combobox_buf;
	my $listbox_buf1;
	my $listbox_buf2;
	my $listbox_buf3;
	my $chromosome_buf;
	my $combobox_options;
	my $listbox_options;
	my $chromosome_options;

	# dbtable: actually it is now a schema

	# If $dbtable is not defined, get default dataset from db
	if($dbtable eq "") {
		my $query = "SELECT msqt_schema_name FROM public.msqt_config WHERE default_dataset = true";
		my $sth = $dbh->prepare( $query );
		$sth->execute() || print "<font color=red><b>" . $sth->errstr . "</b></font><br><br>";
		while ( $row = $sth->fetchrow ) {
			$dbtable = $row;
		}
		$sth->finish;
		
		if($DEBUG) { print "got default dataset from db: $dbtable<br>"; }
	}

	# Determine which database structure is used
	$query = "SELECT msqt_schema_type FROM public.msqt_config WHERE msqt_schema_name = '$dbtable'";
	
	if($DEBUG) { print "query: $query<br>"; }

	$sth = $dbh->prepare( $query );
	$sth->execute() || print "<font color=red><b>" . $sth->errstr . "</b></font><br><br>";
	while ( $row = $sth->fetchrow ) {
		$schema_type = $row;
	}
	if($schema_type eq "struct_position_name") {
		open FORM, $content_file_posname or die $!;
		# read template
		while(<FORM>) {
				$file_buf .= $_;
		}
		close FORM;
	}
	elsif($schema_type eq "struct_position_chromosome") {
		open FORM, $content_file_chrpos or die $!;
		# read template
		while(<FORM>) {
		$file_buf .= $_;
		}
		close FORM;
	}
	else {
		die ("configuration error: no template found for $dbtable ($row).")
	}


	# Insert headline
	$file_buf =~ s/--headline--/MSQT\/<b>SBE<\/b>/;
	
	# Insert scriptloc into FORM tag
	$file_buf =~ s/--MSQT-TEMPL:scriptloc--/$script_loc/;
	

	# Generate tabbar
	my $tabbar_buf;
	$tabbar_buf = <<HTML;
	<div class="tabbar_table">
			<div class="tabbar_tr">
				<span class="tabbar_td"><a class="tabbar_link" href="[msqtwebroot]/index.html">Home</a></span> <span class="toolbar_divider">&nbsp;&nbsp;&nbsp;&nbsp;</span>
				<span class="tabbar_td"><a class="tabbar_link" href="[msqtcgiroot]/msqt-sbe.cgi">SNPs Between Ecotypes</a></span> <span class="toolbar_divider">&nbsp;&nbsp;&nbsp;&nbsp;</span>
				<span class="tabbar_td"><a class="tabbar_link" href="[msqtcgiroot]/msqt-adf.cgi">Assay Development Formater</a></span> <span class="toolbar_divider">&nbsp;&nbsp;&nbsp;&nbsp;</span>
				<span class="tabbar_td"><a class="tabbar_link" href="[msqtcgiroot]/msqt-e.cgi">Expert Mode</a></span> <span class="toolbar_divider">&nbsp;&nbsp;&nbsp;&nbsp;</span>
				<span class="tabbar_td"><a class="tabbar_link" href="[msqtcgiroot]/msqt-sniped.cgi">SNIPED!</a></span> <span class="toolbar_divider">&nbsp;&nbsp;&nbsp;&nbsp;</span>
				<span class="tabbar_td"><a class="tabbar_link" href="[msqtcgiroot]/msqtadmin/msqtadmin.cgi" target="__msqtadmin__">Admin</a></span> <span class="toolbar_divider">&nbsp;&nbsp;&nbsp;&nbsp;</span>
				<span class="tabbar_td"><a class="tabbar_link" href="[msqtwebroot]/doc.html">Documentation</a></span> <span class="toolbar_divider">&nbsp;&nbsp;&nbsp;&nbsp;</span>
				<span class="tabbar_td"><a class="tabbar_link" href="[msqtwebroot]/about.html">About</a></span>
			</div>
		</div>
HTML
	$file_buf =~ s/<!--MSQT-TEMPL:tabbar-->/$tabbar_buf/;
	

	# generate schema combobox
	# query existing schemas from database
	$query = "SELECT msqt_schema_name FROM public.msqt_config";
	if($DEBUG) { print "query: $query<br>"; }
	$sth = $dbh->prepare( $query );
	$sth->execute() || print "<font color=red><b>" . $sth->errstr . "</b></font><br><br>";

        # generate options for combobox
	my $row_cnter=0;
	my @row_arr;
	my $default_dataset;
	while ( @row_arr = $sth->fetchrow_array ) {
		$combobox_options .= "<option value=\"" . $row_arr[0] . "\"";
		$combobox_options .= ">$row_arr[0]</option>\n";
		$row_cnter++;
	}
	$sth->finish();

	# generate combobox
	$combobox_buf = "<select name=\"sbe_database\" tabindex=\"1\">";
	$combobox_buf .= $combobox_options;
	$combobox_buf .= "</select>";

	# select right entry
	if($DEBUG) { print "dbtable: $dbtable<br>"; }
	$combobox_buf =~ s/value="$dbtable"/value="$dbtable" selected/;


	# insert combobox into template
	$file_buf =~ s/<!--schema_list-->/$combobox_buf/;



	# generate ecotypes listboxes
	# query ecotype lists from database
	$query = "SELECT * FROM $dbtable.ecotypes WHERE ecotype !='target'";

	if($DEBUG) { print "query: $query<br>"; }
	$sth = $dbh->prepare( $query );
	$sth->execute() || print "<font color=red><b>" . $sth->errstr . "</b></font><br><br>";

	# generate options for listboxes
	my $row_cnter=0;
	while ( $row = $sth->fetchrow ) {
		$listbox_options .= "<option value=\"" . $row . "\">$row</option>\n";
		$row_cnter++;
	}
	$sth->finish();

	# generate listboxes

	$listbox_buf1 = "<select name=\"ecot_list_1\" size=\"15\" multiple=\"multiple\">";
	$listbox_buf1 .= $listbox_options;
	$listbox_buf1 .= "</select>";


	$listbox_buf2 = "<select name=\"ecot_list_2\" size=\"15\" multiple=\"multiple\">";
	$listbox_buf2 .= $listbox_options;
	$listbox_buf2 .= "</select>";

	$listbox_buf3 = "<select name=\"ecot_list_3\" size=\"15\" multiple=\"multiple\">";
	$listbox_buf3 .= $listbox_options;
	$listbox_buf3 .= "</select>";


        # select the right ecotypes in listbox
        foreach my $entry (split(/,/, $ecot_list1)) {
          $entry =~ s/'//g;
          if( !grep( /^$entry$/, $listbox_buf1) ) {
            $listbox_buf1 =~ s/value="$entry"/value="$entry" selected/;
          }
        }
        foreach my $entry (split(/,/, $ecot_list2)) {
          $entry =~ s/'//g;
          if( !grep( /^$entry$/, $listbox_buf2) ) {
            $listbox_buf2 =~ s/value="$entry"/value="$entry" selected/;
          }
        }
        foreach my $entry (split(/,/, $ecot_list3)) {
          $entry =~ s/'//g;
          if( !grep( /^$entry$/, $listbox_buf3) ) {
            $listbox_buf3 =~ s/value="$entry"/value="$entry" selected/;
          }
        }


	# insert listboxes into template
	$file_buf =~ s/<!--ecotype_list_1-->/$listbox_buf1/;
	$file_buf =~ s/<!--ecotype_list_2-->/$listbox_buf2/;
	$file_buf =~ s/<!--ecotype_list_3-->/$listbox_buf3/;

	# insert results into template
	$file_buf =~ s/<!--results-->/$html_results/;


        # fill out other fields
        if($schema_type eq "struct_position_name") {
          # frag_id
          if($frag_id) {
            $file_buf =~ s/--frag_id_value--/$frag_id/;
          }
          else {
            $file_buf =~ s/--frag_id_value--//;
          }

          # pos_name
          if($pos_name) {
            $file_buf =~ s/--pos_name_value--/$pos_name/;
          }
          else {
            $file_buf =~ s/--pos_name_value--//;
          }
        }
        elsif($schema_type eq "struct_position_chromosome") {
          # start position
          if($start_pos) {
            $file_buf =~ s/--start_pos_value--/$start_pos/;
          }
          else {
            $file_buf =~ s/--start_pos_value--/0/;
          }

          # end position
          if($end_pos) {
            $file_buf =~ s/--end_pos_value--/$end_pos/;
          }
          else {
            $file_buf =~ s/--end_pos_value--/0/;
          }

          #chromosome

          # generate chromosome list
          # query number of chromosomes from database
          $query = "SELECT chromosomes FROM public.msqt_config WHERE msqt_schema_name='$dbtable'";
          if($DEBUG) { print "query: $query<br>"; }
          $sth = $dbh->prepare( $query );
          $sth->execute() || print "<font color=red><b>" . $sth->errstr . "</b></font><br><br>";
          my @row_chromosomes = $sth->fetchrow_array;
          $sth->finish();

          # generate options for chromosome list
          for(my $i=1; $i<=$row_chromosomes[0];$i++) {
                  $chromosome_options .= "<option value=\"" . $i . "\">$i</option>\n";
          }

          # generate chromosome list
          $chromosome_buf = "<select name=\"chromosome\" tabindex=\"3\">";
          $chromosome_buf .= $chromosome_options;
          $chromosome_buf .= "</select>";

          # select right entry
          if($dbtable) {
            $chromosome_buf =~ s/value="$chromosome"/value="$chromosome" selected/;
          }

          # insert chromosome list into template
          $file_buf =~ s/<!--chromosome_list-->/$chromosome_buf/;

        }



	print $file_buf;
	close FORM;
}


sub parse_params {
	my $q = new CGI;
	my( $name, $wert );
	my $buf = "";		# buffer for db results
	my $header_str;		# result header


	# get the selected ecotypes for list 1
	foreach $name ( $q->param ) {
		if($name =~ /ecot_list_1/) {
			if($DEBUG) { print "found $name<br>"; }
			foreach $wert ( $q->param( $name ) ) {
				$ecot_list1 .= "'$wert',";
			}
		}
	}
	chop($ecot_list1); # remove last ","

	# get the selected ecotypes for list 2
	foreach $name ( $q->param ) {
		if($name =~ /ecot_list_2/) {
			if($DEBUG) { print "found $name<br>"; }
			foreach $wert ( $q->param( $name ) ) {
				$ecot_list2 .= "'$wert',";
			}
		}
	}
	chop($ecot_list2); # remove last ","

	# get the selected ecotypes for list 2
	foreach $name ( $q->param ) {
		if($name =~ /ecot_list_3/) {
			if($DEBUG) { print "found $name<br>"; }
			foreach $wert ( $q->param( $name ) ) {
				$ecot_list3 .= "'$wert',";
			}
		}
	}
	$ecot_list3 .= "'target',"; # add target
	chop($ecot_list3); # remove last ","

if($DEBUG) { print "*parse_params: read ecot_list from GUI <br>";
print "ecot_list1:$ecot_list1<br>ecot_list2:$ecot_list2<br>ecot_list3:$ecot_list3<br><br>"; }

	# check action and if($DEBUG) { print "query: $query"; }perform db qureies
	if( defined $q->param( "change_db" ) ) {
		$dbtable 	= $q->param( "sbe_database" );
		$html_results = "Database changed to '$dbtable'.";

	}
	if( defined $q->param( "compute_snps" ) ) {
		if($DEBUG) { print "compute_snps called with<br>"; }

                # We have two different GUIs:
                # 1. for data with chromosome/position structure
                # 2. for data with position_name/fragid structure
                # Now, get the type:
                $dbtable = $q->param( "sbe_database" );
                $query = "SELECT msqt_schema_type FROM public.msqt_config WHERE msqt_schema_name = '$dbtable'";
                if($DEBUG) { print "query: $query<br>"; }

                $sth = $dbh->prepare( $query );
                $sth->execute() || print "<font color=red><b>" . $sth->errstr . "</b></font><br><br>";
                while ( $row = $sth->fetchrow ) {
                  $schema_type = $row;
                }

                if($DEBUG) { print "compute_snps: schema_type=$schema_type<br>"; }
                if($schema_type eq "struct_position_name") {
                  $frag_id = $q->param( "frag_id" );
                  $pos_name = $q->param( "pos_name" );
                  if($DEBUG) { print "Got param frag_id: $frag_id<br>"; }
                  if($DEBUG) { print "Got param pos_name: $pos_name<br>"; }
                }
                elsif($schema_type eq "struct_position_chromosome") {
                  $start_pos = $q->param( "start_pos" );
                  $end_pos = $q->param( "end_pos" );
                  $chromosome = $q->param( "chromosome" );
                }
                else {
                  die "schema type undefined for database '$dbtable'";
                }

		if($DEBUG) { print "$start_pos, $end_pos, $chromosome, $ecot_list1, $ecot_list2, $dbtable<br>\n"; }


                # Do computation
                if($schema_type eq "struct_position_name") {
                  snps_between_ecotypes_posname();
                }
                elsif($schema_type eq "struct_position_chromosome") {
                  snps_between_ecotypes_chrpos();
                }
         }
}


# called by parse_params()
sub snps_between_ecotypes_posname {

	# Parameters submitted by GUI
	# string containing ecotypes must be converted into hash
	my %ethash;
	$ecot_list1 =~ s/'//g;
	$ecot_list2 =~ s/'//g;
	$ecot_list3 =~ s/'//g;

if($DEBUG) { print "*snps_between_ecotypes_posname: read ecot_list from GUI <br>";
print "ecot_list1:$ecot_list1<br>ecot_list2:$ecot_list2<br>ecot_list3:$ecot_list3<br><br>"; }

	foreach my $et (split(",", $ecot_list1)) {
		$ethash{$et} = 0;
	}
	foreach my $et (split(",", $ecot_list2)) {
		$ethash{$et} = 1;
	}
	foreach my $et (split(",", $ecot_list3)) {
		$ethash{$et} = 2;
	}
	
	# Check if the ecotype list is distinct. 
	#If not, report an error, distinct_snps _needs_ distinct ecotype groups
	my ($c0, $c1, $c2) = (0,0,0);
	my ($key, $value);
	while(($key, $value) = each(%ethash)) {
		$c0=1 if(($value==0) && ($c0==0));
		$c1=1 if(($value==1) && ($c1==0));
		$c2=1 if(($value==2) && ($c2==0));
	}
	if(($c0+$c1+$c2)!=3) {
		$html_results .= "<b style=\"color: red;\">An error occured: The ecotype sets must be disjoint</b>";
		return;
	}
	
	
if($DEBUG) {  print "*snps_between_ecotypes_posname: ecotype-hash: <br>";
foreach my $ecotype (keys %ethash){
     print"$ethash{$ecotype} - $ecotype<br>";
}
}

	$html_results  = "Query initiated with following parameters:<br><br>";
	$html_results .= "frag_id    = $frag_id<br>";
	$html_results .= "pos_name   = $pos_name<br>";
	$html_results .= "ecot_list1 = $ecot_list1<br>";
	$html_results .= "ecot_list2 = $ecot_list2<br>";
	$html_results .= "also annotated: $ecot_list3<br><br><p>";


	$html_results .= "\n<table border=0 cellspacing=0>";
	$html_results .= "<tr bgcolor=\"lightgrey\">";
	$html_results .= "<td class=\"msqttableresult\"><b>Type</b></td>\n";
	$html_results .= "<td class=\"msqttableresult\"><b>Pos</b></td>\n";
	$html_results .= "<td class=\"msqttableresult\"><b>Allele1</b></td>\n";
	$html_results .= "<td class=\"msqttableresult\"><b>Allele2</b></td>\n";
	$html_results .= "<td class=\"msqttableresult\"><b>FragID</b></td>\n";
	$html_results .= "<td class=\"msqttableresult\"><b>Alivie Link</b></td>\n";
	$html_results .= "<td class=\"msqttableresult\"><b>ADF Link</b></td>\n";
	$html_results .= "</tr>\n";
	
	if($DEBUG) { print "DEBUG: snps_between_ecotypes_posname: dbtable: $dbtable  "; }

	my $snp_list = new distinct_snps_posname();
	# Initialize snp_lists with known start and end
	if($start_pos>$end_pos) {
		($start_pos,$end_pos) = ($end_pos,$start_pos);
	}

        if($DEBUG) {
	print "<br>DEBUG: msqt-sbe.cgi: snp_list->init(): ";
	print "   chr=$chromosome, start_pos=$start_pos, start_pos=$end_pos, table=\"$dbtable.snps\"<br>";
        }

	#$snp_list->init($chromosome, $start_pos, $end_pos, $dbh, $dbtable, %ethash);
	$snp_list->init($chromosome, $frag_id, $pos_name, $dbh, $dbtable, %ethash);

	
	# Print clustered primary SNPs
	my $xml_results;
	# Print clustered primary SNPs
	foreach (@{$snp_list->{output}}) {
		$xml_results .= "$_";
	}
	# parse XML output
	my $xs = new XML::Simple();
	my $sbe = $xs->XMLin($xml_results, forcearray => ['polymorphism']);

	
	foreach my $polym (@{$sbe->{polymorphism}}) {
		
		$html_results .= "<tr>\n";
		
		$html_results .= "<td class=\"msqttableresult\">$polym->{type}</td>\n";
		$html_results .= "<td class=\"msqttableresult\">$polym->{position}</td>\n";
		$html_results .= "<td class=\"msqttableresult\">$polym->{allele1}</td>\n";
		$html_results .= "<td class=\"msqttableresult\">$polym->{allele2}</td>\n";
		$html_results .= "<td class=\"msqttableresult\">$polym->{fragid}</td>\n";
		
		
		# link definition for alivie
		$html_results .= "<td class=\"msqttableresult\"><a href=\"javascript: openWindow("
				. "'$alivie_cgi?alivie=alivie&dbtable=$dbtable&chr=$chromosome&frag_id="
				. $polym->{fragid}
				. "','"
				. $polym->{fragid}
				."','830','560')\""
				. "\">"
				. "alignment"
				. "</a>"
				. "</td>";
		# link definition for adf
		$html_results .= "<td class=\"msqttableresult\"><a href=\"$adf_cgi"
				. "?sbe_database=$dbtable"
				. "&ecot_list_1=" . join("&ecot_list_1=",split(",",$ecot_list1))
				. "&ecot_list_2=" . join("&ecot_list_2=",split(",",$ecot_list2))
				. "&ecot_list_3=" . join("&ecot_list_3=",split(",",$ecot_list3))
				. "&pos_name="
				. $polym->{position}
				. "&frag_id="
				. $polym->{fragid}
				. "&chromosome=$chromosome&compute_snps=Compute+SNPs\">"
				. "assay_development_format</a></td>";	
				
		$html_results .= "</tr>\n";
	}
	$html_results .= "</table>";
	$html_results .= "<br><p>ready.";
}

sub snps_between_ecotypes_chrpos {

	# Parameters submitted by GUI
	# string containing ecotypes must be converted into hash
	my %ethash;
	$ecot_list1 =~ s/'//g;
	$ecot_list2 =~ s/'//g;
	$ecot_list3 =~ s/'//g;

	foreach my $et (split(",", $ecot_list1)) {
		$ethash{$et} = 0;
	}
	foreach my $et (split(",", $ecot_list2)) {
		$ethash{$et} = 1;
	}
	foreach my $et (split(",", $ecot_list3)) {
		$ethash{$et} = 2;
	}


	# Check if the ecotype list is distinct. 
	#If not, report an error, distinct_snps _needs_ distinct ecotype groups
	my ($c0, $c1, $c2) = (0,0,0);
	my ($key, $value);
	while(($key, $value) = each(%ethash)) {
		$c0=1 if(($value==0) && ($c0==0));
		$c1=1 if(($value==1) && ($c1==0));
		$c2=1 if(($value==2) && ($c2==0));
	}
	if(($c0+$c1+$c2)!=3) {
		$html_results .= "<b style=\"color: red;\">An error occured: The ecotype sets must be disjoint</b>";
		return;
	}
	
	
	$html_results  = "Query initiated with following parameters:<br><br>";
	$html_results .= "chromosome = $chromosome<br>";
	$html_results .= "start_pos  = $start_pos<br>";
	$html_results .= "end_pos    = $end_pos<br>";
	$html_results .= "ecot_list1 = $ecot_list1<br>";
	$html_results .= "ecot_list2 = $ecot_list2<br>";
	$html_results .= "also annotated: $ecot_list3<br><br><p>";

	$html_results .= "\n<table border=0 cellspacing=0>";
	$html_results .= "<tr bgcolor=\"lightgrey\">";
	$html_results .= "<td class=\"msqttableresult\"><b>Chr</b></td>\n";
	$html_results .= "<td class=\"msqttableresult\"><b>Type</b></td>\n";
	$html_results .= "<td class=\"msqttableresult\"><b>Pos</b></td>\n";
	$html_results .= "<td class=\"msqttableresult\"><b>Allele1</b></td>\n";
	$html_results .= "<td class=\"msqttableresult\"><b>Allele2</b></td>\n";
	$html_results .= "<td class=\"msqttableresult\"><b>FragID</b></td>\n";
	$html_results .= "<td class=\"msqttableresult\"><b>Alivie Link</b></td>\n";
	$html_results .= "<td class=\"msqttableresult\"><b>ADF Link</b></td>\n";
	$html_results .= "</tr>\n";
	
	if($DEBUG) { print "dbtable: $dbtable  "; }

	my $snp_list = new distinct_snps_chrpos();
	# Initialize snp_lists with known start and end
	if($start_pos>$end_pos) {
		($start_pos,$end_pos) = ($end_pos,$start_pos);
	}
	$snp_list->init($chromosome, $start_pos, $end_pos, $dbh, $dbtable, %ethash);

	# Initialize snp_list for a special position (and its fragment)
	#$snp_list->find_frag($chr, $position, $dbh, $dbtable, %ecotypes);

	# Create genaissance output for fragment
	#$snp_list->genaissance($dbh);

	# Print clustered primary SNPs
	my $xml_results;
	# Print clustered primary SNPs
	foreach (@{$snp_list->{output}}) {
		$xml_results .= "$_";
	}
	# parse XML output
	my $xs = new XML::Simple();
	my $sbe = $xs->XMLin($xml_results, forcearray => ['polymorphism']);


	foreach my $polym (@{$sbe->{polymorphism}}) {
	
		$html_results .= "<tr>\n";
		
		$html_results .= "<td class=\"msqttableresult\">$polym->{chromosome}</td>\n";
		$html_results .= "<td class=\"msqttableresult\">$polym->{type}</td>\n";
		$html_results .= "<td class=\"msqttableresult\">$polym->{position}</td>\n";
		$html_results .= "<td class=\"msqttableresult\">$polym->{allele1}</td>\n";
		$html_results .= "<td class=\"msqttableresult\">$polym->{allele2}</td>\n";

		
		# highlight fragid if primer exists
		if($polym->{hasprimer} eq "true") {
			$html_results .= "<td class=\"msqttableresult\"><font style=\"background-color:rgb(255, 186, 66);\">",
					$polym->{fragid} . "</font></td>";
		}
		else {
			$html_results .= "<td class=\"msqttableresult\">" . $polym->{fragid} ."</td>";
		}
		
		# link definition for alivie
		$html_results .= "<td class=\"msqttableresult\"><a href=\"javascript: openWindow("
				. "'$alivie_cgi?alivie=alivie&dbtable=$dbtable&chr=$chromosome&frag_id="
				. $polym->{fragid}
				. "','"
				. $polym->{fragid}
				."','830','560')\""
				. "\">"
				. "alignment"
				. "</a>"
				. "</td>";
		# link definition for adf
		$html_results .= "<td class=\"msqttableresult\"><a href=\"$adf_cgi"
				. "?sbe_database=$dbtable"
				. "&ecot_list_1=" . join("&ecot_list_1=",split(",",$ecot_list1))
				. "&ecot_list_2=" . join("&ecot_list_2=",split(",",$ecot_list2))
				. "&ecot_list_3=" . join("&ecot_list_3=",split(",",$ecot_list3))
				. "&position="
				. $polym->{position}
				. "&chromosome=$chromosome&compute_snps=Compute+SNPs\">"
				. "assay_development_format</a></td>";	
				
		$html_results .= "</tr>\n";
	}
	$html_results .= "</table>\n";
	
	$html_results .= "<br><p>ready.";
}

sub db_connect_error {
	open FORM, $error_file or die $!;
	# read template
	while(<FORM>) {
		print $_;
	}
	exit(0);
}
