#! /usr/bin/perl -w

#
# msqtadmin.cgi
#
# Author: Joffrey Fitz <fitzj@informatik.uni-tuebingen.de>
#
# Copyright (c) 2007 Max Planck Institute for Developmental Biology, 
# 	http://www.eb.tuebingen.mpg.de
#
# This program is part of the MSQT project by Norman Warthmann, Joffrey Fitz 
# and Detlef Weigel (http://msqt.weigelworld.org)
#
# This program is free software; you can redistribute it and/or modify it                        
# under the terms of the GNU General Public License as published by the Free                     
# Software Foundation; either version 2, or (at your option) any later 
# version.
#
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
# or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
# for more details.
#


$DEBUG = 0;
$VERSION = "0.7.3 (Mon Oct 15 2007)";

use CGI qw(:standard);
use CGI::Carp qw( fatalsToBrowser );

use DBI;
use SQLQuery2HTMLTable;

use File::Basename;
use Archive::Extract;

use Digest::MD5 'md5_hex';

    
my $msqtadmin_cgi_root = "[msqtcgiroot]/msqtadmin";

my $archive_dir = "/tmp/msqt_archives";
my $extract_dir = "/tmp/msqt_extracts";
my $fasta_files_dir = "[webrootpath]/fasta_files/";
	
# my $path_to_sql_templates = "$msqtadmin_cgi_root/msqtadmin/msqt_dbsetup/";
my $path_to_sql_templates = "./";

my $sniped_stream_cgi = "[msqtcgiroot]/msqtadmin/sniped_stream2.cgi";

my $target_ecotype_name = "target";
my $chromosome_prefix = "chromosome_";

# database config
my $dbname = "[DBNAME]";
my $host = "[DBHOST]";
my $port = "[DBPORT]";
my $options = "";
my $username = "msqt";
my $password = "[DBPASS]";

my $dbtable = "public.msqt_config";
my $script_loc = "msqtadmin.cgi";

my $dbh;
$dbh = DBI->connect("dbi:Pg:dbname=$dbname;host=$host;port=$port;".
	"", "$username", "$password",
        {AutoCommit => 1} ) || die "Can not connect to database server";


# variables used by process_data
my $dataset_name;
my $dataset_type;
my $dataset_chrom_num;
my $data_path;
my $file;
	

my $cgi = CGI->new;

# Template
my $tmpl_file = "../html/tmpl_msqt-admin.html";
my $tmpl_buf;

open(FH, $tmpl_file) || die "Can not open '$tmpl_file': $!";
while(<FH>) {
	$tmpl_buf .= $_;
}
close(FH);


#
# User auth
#

###
### STORE THIS IN DATABASE !!!
my $user = "[MSQTADMINUSER]";
my $pass = "[MSQTADMINPASS]";
my $md5hash = md5_hex("$user.$pass");
###
###


my $cookname = "msqtadminauth";
my $user_param = param("user");
my $pass_param = param("pass");

my $logout_action = param("action");
my $sid = cookie($cookname) || undef;

if( ($user_param ne "") and ($pass_param ne "") ) { # got user/pass
	if(md5_hex("$user_param.$pass_param") ne $md5hash) { # pass not ok
		print header();
		print_auth_form("Wrong password");
	}
	else {
		my $cookie = cookie(
                -NAME    => $cookname,
                -VALUE   => md5_hex("$user_param.$pass_param"),
                -EXPIRES => "+6h",
            );
		print header(-COOKIE => $cookie, -EXPIRES => "-1d");
	}
}
elsif($sid eq "") { # no cookie
	print header();
 	print_auth_form();
}
elsif($sid ne $md5hash) {
	print header();
	print_auth_form();
}

else {
	if($logout_action eq "logout") {
		my $cookie = cookie(
					-NAME    => $cookname,
					-VALUE   => '',
					-EXPIRES => "-1d",
				);
		print header(-COOKIE => $cookie);
		print_auth_form();
		exit;
	}
	else {
		print header(-EXPIRES => "-1d");
	}
}


#
# Do HTML stuff
my $html_buf;

my $tabbar_buf;
$tabbar_buf = <<HTML;
<div class="toolbar">
	<div class="tabbar_table">
		<div class="tabbar_tr">
			<span class="tabbar_td"><a class="tabbar_link" href="?tab=Main">Home</a></span> <span class="toolbar_divider">&nbsp;&nbsp;&nbsp;&nbsp;</span>
			<span class="tabbar_td"><a class="tabbar_link" href="?tab=Schema Manager">Dataset Manager</a></span> <span class="toolbar_divider">&nbsp;&nbsp;&nbsp;&nbsp;</span>
			<span class="tabbar_td"><a class="tabbar_link" href="?tab=Upload data">Data Uploader</a></span> <span class="toolbar_divider">&nbsp;&nbsp;&nbsp;&nbsp;</span>
			<span class="tabbar_td"><a class="tabbar_link" href="?tab=Snipe4SNPs">Snipe4SNPs</a></span> <span class="toolbar_divider">&nbsp;&nbsp;&nbsp;&nbsp;</span>
			<span class="tabbar_td"><a class="tabbar_link" href="?tab=Server Info">Server Info</a></span> <span class="toolbar_divider">&nbsp;&nbsp;&nbsp;&nbsp;</span>
			<span class="tabbar_td"><a class="tabbar_link" href="[msqtwebroot]/msqt_admin_manual.html">Documentation</a></span> <span class="toolbar_divider">&nbsp;&nbsp;&nbsp;&nbsp;</span>
			<span class="tabbar_td"><a class="tabbar_link" href="?tab=About">About</a></span>  <span class="toolbar_divider">&nbsp;&nbsp;&nbsp;&nbsp;</span>
			<span class="tabbar_td"><a class="tabbar_link" href="?action=logout">Logout</a></span>
		</div>
	</div>
</div>
HTML


#
# Process CGI action
#

print "action: " . $cgi->param('action') . "<br>" if($DEBUG);
if( $cgi->param('action') eq "upload") {
	upload_archive();
	display_html_page();
	exit;
}
elsif( $cgi->param('action') eq "process_data") {
	$dataset_name		= $cgi->param('dataset_name');
	$dataset_type		= $cgi->param('dataset_type');
	$dataset_chrom_num 	= $cgi->param('chromosome_num');
	$data_path 			= $cgi->param('data_path');
	$archive_name 		= $cgi->param('archive_name');
	$file;
	
	process_data();
	display_html_page();
	exit;
}
elsif( $cgi->param('action') eq "delete_schema_confirm") {
	my $schema = $cgi->param('schema');
	delete_schema_confirm($schema);
	display_html_page();
	exit;
}
elsif( $cgi->param('action') eq "delete_schema") {
	my $schema = $cgi->param('schema');
	delete_schema($schema);
}
elsif( $cgi->param('action') eq "set_default_schema") {
	my $schema = $cgi->param('schema');
	set_default_schema($schema);
	show_schema_table();
}
elsif( $cgi->param('action') eq "snipe4snps") {
	my $schema = $cgi->param('schema');
	my $schema_type = $cgi->param('schema_type');
	show_snipe4snps_parameters($schema, $schema_type);
	display_html_page("no_navigation");
	exit;
}
elsif( $cgi->param('action') eq "show_snipe4snps_parameters") {
	my $schema = $cgi->param('schema');
	my $schema_type = $cgi->param('schema_type');
	print "in show_snipe4snps_parameters<br>" if($DEBUG);
	show_snipe4snps_parameters($schema, $schema_type);
	display_html_page();
	exit;
}
elsif( $cgi->param('action') eq "confirm_snip4snps_parameter") {
	my $schema = $cgi->param('schema');
	my $schema_type = $cgi->param('schema_type');
	my $div_threshold = $cgi->param('div_threshold');
	my $right_neigbh_threshold = $cgi->param('right_neigbh_threshold');
	my $left_neigbh_threshold = $cgi->param('left_neigbh_threshold');
	
	show_snipe4snps_confirm($schema, $schema_type, $div_threshold, $right_neigbh_threshold, $left_neigbh_threshold);
	display_html_page();
	exit;
}
elsif( $cgi->param('action') eq "run_snipe4snps") {
	my $schema = $cgi->param('schema');
	my $schema_type = $cgi->param('schema_type');
	my $div_threshold = $cgi->param('div_threshold');
	my $right_neigbh_threshold = $cgi->param('right_neigbh_threshold');
	my $left_neigbh_threshold = $cgi->param('left_neigbh_threshold');
	
	run_snipe4snps($schema, $schema_type, $div_threshold, $right_neigbh_threshold, $left_neigbh_threshold); 
	display_html_page("no_navigation");
	exit;
}
elsif( $cgi->param('action') eq "s4s_empty_confirm") {
	my $schema = $cgi->param('schema');
		
	s4s_empty_confirm($schema); 
	display_html_page();
	exit;
}
elsif( $cgi->param('action') eq "s4s_empty") {
	my $schema = $cgi->param('schema');
		
	s4s_empty($schema); 
}


#
# Process tab change 
#
print "tab: " . $cgi->param('tab') . "<br>"  if($DEBUG);
if( $cgi->param('tab') eq "Main") {
	show_main();
}
elsif( $cgi->param('tab') eq "Schema Manager" ) {
	show_schema_table();
}
elsif( $cgi->param('tab') eq "Upload data" ) {
	show_upload_form();
}
elsif( $cgi->param('tab') eq "Snipe4SNPs" ) {
	show_snipe4snps();
}
elsif( $cgi->param('tab') eq "Server Info" ) {
	use Sys::Hostname;
	$webserver_hostname = hostname();
	$webserver_software =  $ENV{SERVER_SOFTWARE};
	$dbserver_software = $dbh->get_info( 17 ) or die;
	$dbserver_version = $dbh->get_info( 18 );
	
	$html_buf = print_headline("Server Information");
	$html_buf .= <<EOF;
	<table border=0>
	<tr>
	<td class="server_info_tags">Webserver host:</td><td class="server_info_values">$webserver_hostname</td></tr>
	<tr><td class="server_info_tags">Webserver software:</td><td class="server_info_values">$webserver_software</td></tr>
	<tr><td class="server_info_tags">Database server software:</td><td class="server_info_values">$dbserver_software</td></tr>
	<tr><td class="server_info_tags">Database server version:</td><td class="server_info_values">$dbserver_version</td></tr>
	<tr><td class="server_info_tags">Database server host:</td><td class="server_info_values">$host</td></tr>
	<tr><td class="server_info_tags">Database server port:</td><td class="server_info_values">$port</td></tr>
	<tr><td class="server_info_tags">Database name:</td><td class="server_info_values">$dbname</td></tr>
	<tr><td class="server_info_tags">Database user:</td><td class="server_info_values">$username</td></tr>
	<tr><td class="server_info_tags">MSQT configuration table:</td><td class="server_info_values">$dbtable</td></tr>
	</table>
	
EOF
	



}
elsif( $cgi->param('tab') eq "Documentation" ) {
	show_documentation();
}
elsif( $cgi->param('tab') eq "About" ) {
	show_about();
}
else {
	show_main();
}

display_html_page();
# print "</body></html>";
exit;


#############################################################################################################

sub display_html_page {
	my $print_no_navbar = shift;
	$tmpl_buf =~ s#<!--MSQT-TEMPL:tabbar-->#$tabbar_buf# unless($print_no_navbar eq "no_navigation");
	$tmpl_buf =~ s#<!--MSQT-TEMPL:content-->#$html_buf#;
	print $tmpl_buf;
}

#############################################################################################################

sub print_headline {
	my $title = shift;
	return "<h1>$title</h1>";
}

#############################################################################################################

sub print_auth_form {
	my $error = shift;
# 	print h1("Auth"),
#           hr(), start_form(),
#             p("Username/Passwd: ", textfield("user",$user_param), textfield("pass",$pass_param)),
#               submit(), end_form(), hr();
	$html_buf = h1("Authentication required").start_form().p("Username: ".textfield("user",$user_param).
				p("Password: ").textfield("pass",$pass_param)).submit().end_form();
				
	$html_buf =<<EOT;
	<h1>Access restriction</h1>
	
	
	<div id="code">
	<h2 class="title">Authentication required:</h2>
	
	<form action="$script_loc" method="post">
	<table>
	<tr>
		<td>Username:  
		</td>
		<td><input type="text" name="user" value="$user_param">
		</td>
	</tr>
	
	<tr>
		<td>Password:  
		</td>
		<td><input type="password" name="pass" value="$pass_param">
		</td>
	</tr>
	
	<tr>
		
		<td><br><br><input type="submit" name="submit" value=" OK ">
		</td>
		<td>
		</td>
	</tr>
	
	</table>
	
	</form>
	</div>

EOT
    if($error) {
		$html_buf .= p(b($error));
	}
	
	display_html_page("no_navigation");
    exit;
}

#############################################################################################################

sub show_main {
	$html_buf = <<EOT;
		
	<h1>Welcome to MSQT/Admin!</h1>
	<div class="doc_container">
	
	With this tool you can manage your MSQT datasets.<br>
	For further information please consult the <a href="$script_loc?tab=Documentation">documentation</a>.
	<p><br>
	<ul>
		<li><a href="$script_loc?tab=Schema Manager">Dataset Manager</a>: Delete datasets and define the default dataset</li><br>
		<li><a href="$script_loc?tab=Upload data">Data Uploader</a>: Import your data into the database</li><br>
		<li><a href="$script_loc?tab=Snipe4SNPs">Snipe4SNPs</a>: Precompile SNPs and empty Sniped! tables</li><br>
	<ul>
	
	</div>
EOT
}

#############################################################################################################

sub show_schema_table {

	$html_buf = print_headline("MSQT Datasets");
	
	my $query = <<EOQ;
SELECT msqt_schema_name AS "dataset name", 
msqt_schema_type AS "Type", 
chromosomes, 
uploaded_archive AS "Uploaded archive", 
population_date AS "Population date", 
default_dataset AS "Default dataset"
FROM $dbtable ORDER BY msqt_schema_name;
EOQ
	
	my $table4 = new SQLQuery2HTMLTable;
	
	my $tablestyle		  = "width:100%; border-collapse:normal; ";
	my $headercellstyle	  = "background:#D3D4DA; text-align:left; white-space:nowrap;";
	my $evenrowstyle      = "background-color: #E4EAEF; color:black; vertical-align:top";
	my $oddrowstyle       = "background-color: #F5F5F5; color:black; vertical-align:top";
	my $evenrowstylehover = "background-color: #77B1E2; color:black; vertical-align:top";
	my $oddrowstylehover  = "background-color: #77B1E2; color:black; vertical-align:top";
	
	# 	my $front_extension_content = "<a href=\"$script_loc?viewfasta=__COL0__\" target=\"__aaces_fasta__COL0____COL1__\">View FASTA</a>";
	my $back_extension_content 	= "<div style=\"white-space:nowrap\"><a href=\"$script_loc?tab=Schema Manager&action=delete_schema_confirm&schema=__COL0__\">delete</a>"; # __COLn__ will be substitued to the value of colmun n in row k
	$back_extension_content .= "&nbsp;&nbsp;|&nbsp;&nbsp;<a href=\"$script_loc?tab=Schema Manager&action=set_default_schema&schema=__COL0__\">set default</a></div>";
	
	# 	my $custom_header_template = "<a href=\"$script_loc?search_string=$search_string&search_field=$search_field&row_limit=$row_limit&orderquery=__HEADER_TITLE__&search=Hit+me\">__HEADER_TITLE__</a>";
	
	
# 	my $expr = '$col_content =~ s/(.*)(\.[zy][ab].exp)/<div style="white-space:nowrap"><a href="?viewallreads=$1" target="__aaces_allreads_$1" title="View all available reads for $1">-><-<\/a>&nbsp;&nbsp;$1$2<\/div>/; ';
	my $expr = '$col_content =~ s/0//; $col_content =~ s/1/yes/ ';
	my $expr_chroms = '$col_content =~ s/0/n\/a/;';
	
	my %custom_cells;
	$custom_cells{5} = "$expr";			# Default data set
	$custom_cells{2} = "$expr_chroms";	# Chromosomes
	
	
	$table4->set( 
		dbh => $dbh, query => $query,
		tablestyle => $tablestyle, headercellstyle => $headercellstyle,
		twocolorrows => 1, 
		evenrowstyle => $evenrowstyle, oddrowstyle => $oddrowstyle,
		rowhighlight => 1,
		evenrowstylehover => $evenrowstylehover, oddrowstylehover => $oddrowstylehover,
		back_extension_column => "Action", back_extension_content => $back_extension_content,
		# 		front_extension_column => "Sequence file", front_extension_content => $front_extension_content,
		custom_header => 0,
		# 		custom_header_template => $custom_header_template,
		expr => \%custom_cells
	);
	
	# 	$html_results .= $table4->gen_html_table;
	$html_buf .=  $table4->gen_html_table;
	$html_buf .=  "<p><a href=\"$script_loc?tab=Schema Manager\"><b>Refresh list</b></a>\n";

}

#############################################################################################################

sub delete_schema_confirm {
	my $schema = shift;
	
	$html_buf = print_headline("MSQT Datasets");
	
	# Check: Do not delete a schema that is set to default!
	my $query = "select default_dataset from $dbtable where msqt_schema_name = '$schema'";
	my $sth = $dbh->prepare( $query );
	if( not $sth->execute() ) {
		$html_buf .= "<div style=\"color:red; background:#F5F5F5; border:solid 1px grey;\"><b>ERROR: <i>" . $sth->errstr . "</i></b></div><p>";	
	}
	my $default = ($sth->fetchrow_array)[0];
	if($default) {
		$html_buf .= print_error_request("Do not delete the default schema!");
		return;
	}
	
	$html_buf .= <<EOS;
<div class="error_box">
<div class="error_box_headline">Confirmation Request</div><br>
Do you really want to delete schema <i>$schema</i>?<p>

 
<a href="$script_loc?action=delete_schema&schema=$schema&tab=Schema Manager"><b>Yes</b></a>
<a href="$script_loc?tab=Schema Manager">No</a>

</div>
EOS
}

#############################################################################################################

sub delete_schema {
	my $schema = shift;
	
	$html_buf = print_headline("MSQT Datasets");
	
	my $myerr = "";
	
	my $query = "delete from $dbtable where msqt_schema_name = '$schema';";
	$query .=   "drop schema $schema cascade;";
	
          if($DEBUG) { print "query: $query<br>"; }

	my $sth = $dbh->prepare( $query );
	if( not $sth->execute() ) {
		$html_buf .= "<div style=\"color:red; background:#F5F5F5; border:solid 1px grey;\"><b>ERROR: <i>" . $sth->errstr . "</i></b></div><p>";
		
	} else {
		$html_buf .= <<EOS;
<div style="padding:20px; background:#F5F5F5; border:solid 1px grey;">
<div style="color:green; font-size:biggest; font-weight:bold;">Confirmation Request</div><br>
Schema <i>$schema</i> successfully deleted.<p>

 
<a href="$script_loc?tab=Schema Manager"><b>Return to main page</b></a>

</div>
EOS
	
	}
}

#############################################################################################################

sub set_default_schema {
	my $schema = shift;
	
	my $query = "UPDATE $dbtable SET default_dataset = false; ";
	$query .= "UPDATE $dbtable SET default_dataset = true where msqt_schema_name = '$schema';";
		
		if($DEBUG) { print "query: $query<br>"; }

	my $sth = $dbh->prepare( $query );
	if( not $sth->execute() ) {
		print "<div style=\"color:red; background:#F5F5F5; border:solid 1px grey;\"><b>ERROR: <i>" . $sth->errstr . "</i></b></div><p>";
	}
}

#############################################################################################################

sub show_upload_form {

	$html_buf = print_headline("MSQT Database Population: <i>Step 1 of 3</i>");
	
	$html_buf .= <<EOT;
		 
	<FORM ACTION="$script_loc" METHOD="post" ENCTYPE="multipart/form-data">
	
	<div id="code">
	<h2 class="title">Data upload:</h2>
	
	<table>
	<tr>
	<td>
		Name of dataset:
	</td>
	<td>
		<input type="text" name="dataset_name" id="dataset_name" size="30" value="nojo01">
	</td>
	</tr>
	<tr>
	<td>
		Structure of data:
	</td>
	<td>
		<input type="radio" name="dataset_type" value="posname" checked> Position name<br>
		<input type="radio" name="dataset_type" value="chrpos"> Chromosome position<br>
	</td>
	</tr>
	
	<tr>
	<td>
		Chromosomes:
	</td>
	<td>
		<input type="text" name="chromosome_num" id="chromosome_num" size="2" value="">
	</td>
	</tr>
	
	<tr>
	<td>
		Archive File:
	</td>
	<td>
		<input name="file" type="file" size="20">
	</td>
	</tr>
	</fieldset>
	<p>
	</table>
	
	<p><br><br>
	
	<input type="submit" name="submit_button" value="Upload">
	<input type="hidden" name="action" value="upload">
	<input type="hidden" name="tab" value="Upload data">
	
	</FORM>
	<br>
	
	</div>

EOT

# 	print $html;
}

#############################################################################################################

sub print_error_request {
	my $msg = shift;
	
	return <<EOS;
<div class="error_box">
<div class="error_box_headline">Error Message</div><br>
There are some errors: <b>$msg</b><p>
 
<a href="javascript:history.back()"><b>Back</b></a>

</div>
EOS
	exit 1;
}

#############################################################################################################

sub upload_archive {
	# must be POST in previous form !
	my $dataset_name		= $cgi->param('dataset_name');
	my $dataset_type		= $cgi->param('dataset_type');
	my $dataset_chrom_num 	= $cgi->param('chromosome_num');
	my $file = $cgi->param('file');
	my $name = basename($file);
	
	# Check User input
	if($dataset_type eq "chrpos" && ($dataset_chrom_num eq "" || $dataset_chrom_num < 1) ) {
		$html_buf = print_headline("MSQT Database Population");
		$html_buf .= print_error_request("Chromosome number must be a positive integer when you use 'chrpos' as dataset type.");
		return;
	}

	if($dataset_name eq "") {
		$html_buf = print_headline("MSQT Database Population");
		$html_buf .= print_error_request("No dataset name given.");
		return;
	}
	
	if($file eq "") {
		$html_buf = print_headline("MSQT Database Population");
		$html_buf .= print_error_request("No data file given.");
		return;
	}
	
	if($dataset_type eq "posname") {
		$dataset_chrom_num = 0;
	}
	# Check if schema exists
# 	if( check_msqt_schema() ) {
# 		print_error_request("MSQT schema with name '$dataset_name' allready exists.");
# 	}
	
	### create dir for upload
	my $cmd = "rm -rf $archive_dir/ && mkdir $archive_dir";
	system( $cmd ) == 0 or ($html_buf .= error("system $cmd_rm failed: $?") and return 1);

	open(LOCAL, ">$archive_dir/$name") or ($html_buf .= print_error_request("$archive_dir: " . $!) and return);
	while(<$file>) {
		print LOCAL $_;
	}
	
	### build an Archive::Extract object
	my $ae = Archive::Extract->new( archive => "$archive_dir/$name" );
	
	### remove old extract dir
	my $cmd_rm = "rm -rf $extract_dir/*";
	system( $cmd_rm ) == 0 or ($html_buf .= error("system $cmd_rm failed: $?") and return 1);
	### extract to /tmp
	my $ok = $ae->extract( to => "$extract_dir" ) or ($html_buf .= print_error_request("$ae->error" . $!) and return);
	
	### files from the archive
	my $files   = $ae->files;
	
	### dir that was extracted to
	#my $outdir  = $ae->extract_path;
	
	
	### quick check methods
	my $is_tar = $ae->is_tar;     # is it a .tar file?
	my $is_tgz = $ae->is_tgz;     # is it a .tar.gz or .tgz file?
	my $is_gz  = $ae->is_gz;     # is it a .gz file?
	my $is_zip = $ae->is_zip;    # is it a .zip file?
	my $is_bz2 = $ae->is_bz2;    # is it a .bz2 file?
	my $is_tbz = $ae->is_tbz;    # is it a .tar.bz2 or .tbz file?
	
	$html_buf = print_headline("MSQT Database Population: <i>Step 2 of 3</i>");
		
	$html_buf .= <<EOF;
	<div id="code">
	<h2 class="title">Check your Ecotypes:</h2>
	
	<div class=\"msg_ok_string\">$file has been successfully uploaded and extracted.</div>
EOF
	
	my $data_path = $ae->extract_path;
	print "Extracted to: " . $data_path . "<br>"  if($DEBUG);
	
	
	# Browse extracted files and collect ecotypes
	### TODO: Check for existance of ecotype 'target' ###
	my %distinct_ecotypes;
	my $file_count = 0;
	
	use File::Find;
	find sub { 
		if( -f $File::Find::name) { # skip directories and process only files
	# 		print $File::Find::name,  "<br>";
			$file_count++;

			# Work around for archives created with Mac OSX:
			# OSX includes .-files in the archive, e.G. icons and .DSstore directories
			my $__filename = basename($File::Find::name);
			if($__filename =~ m/^\./) { # delete files beginning with .
				print "DEBUG: deleting $File::Find::name ($__filename contains '.')<br>" if($DEBUG);
				system( "rm $File::Find::name" ) == 0 or die $!;
				return;
			}
			print "DEBUG: opening $File::Find::name<br>" if($DEBUG);

			open(IN, $File::Find::name) or die "Can not open '$File::Find::name': $!";
			while(<IN>) {
				($distinct_ecotypes{$1} = "found") if m/^>(.*)$/; # Create a distinct list of ecotypes
			}
			close(IN);
		} 
	}, $data_path;
	
	
	# Print distinct ecotype list
	$html_buf .= "<div class=\"msg_ok_string\">Found " . keys( %distinct_ecotypes ) . " distinct ecotypes in " . $file_count . " files.</div>";
	
	# Show ecotypes in textarea widget
	$html_buf .= "<p>List of distinct ecotypes:<br>";
	$html_buf .= "<textarea name=\"ecotypes\" cols=40 rows=20 readonly>";
	foreach my $key (sort keys %distinct_ecotypes) {
		$html_buf .= "$key\n";
	}
	$html_buf .= "</textarea><p><br>";
	
	$html_buf .= <<EOF;
	Please confirm the correctness of this list
	
	<p>
	<table border="0"><tr><td>
		<form action="$script_loc" method="post">
		<input type="submit" name="submit" value="Proceed with data processing">
		
		<input type="hidden" name="data_path" value="$data_path">
		<input type="hidden" name="archive_name" value="$name">
		<input type="hidden" name="dataset_name" value="$dataset_name">
		<input type="hidden" name="dataset_type" value="$dataset_type">
		<input type="hidden" name="chromosome_num" value="$dataset_chrom_num">
		<input type="hidden" name="action" value="process_data">
		<input type="hidden" name="tab" value="Upload data">
		</form>
	</td><td>
		<form action="$script_loc" method="post">
		<input type="hidden" name="tab" value="Upload data">
		<input type="submit" name="Cancel" value="Cancel and return to main screen">
		</form>
	</td></tr>
	</table>

	</div>
EOF

}

#############################################################################################################

sub process_data {
	
	$html_buf = print_headline("MSQT Database Population: <i>Step 3 of 3</i>");
	
	$html_buf .= <<EOF;
	<div id="code">
	<h2 class="title">Processing data ...</h2>

EOF
	
	
	### Check schema
	if( check_msqt_schema() ) {
		$html_buf .= error("MSQT dataset with name '$dataset_name' allready exists.");
		return;
	} else {
		### Create schema
		if( create_msqt_schema() ) {
			$html_buf .= error("Can not create MSQT dataset.");
			return;
		} else {
			$html_buf .= "<div class=\"msg_ok_string\">MSQT dataset '$dataset_name' created.</div>"; 
		}
	}
	
	
	
	### Process users FASTA files
	# Check file names: does not contain dashes and/or spaces
	if( check_file_names() ) {
 		$html_buf .= error("File names contain dashes or spaces.");
		return;
	} else {
		$html_buf .= "<div class=\"msg_ok_string\">Checked filenames.</div>";
	}

	# Create a list of distinct ecotypes
# 	my %distinct_ecotypes;
	if( collect_distinct_ecotypes() ) {
 		$html_buf .= error("collect_distinct_ecotypes()");
		return;
	} else {
		$html_buf .= "<div class=\"msg_ok_string\">Found " . keys( %distinct_ecotypes ) . " distinct ecotypes.</div>";
	}

	# Fill up missing ecotypes with Ns in Fasta files
	if( check_for_ecot_completness() ) {
		$html_buf .= error("Filling up missing ecotypes with Ns failed");
		return;
	} else { 
		$html_buf .= "<div class=\"msg_ok_string\">Filled up missing ecotypes with Ns in Fasta files.</div>";
	}

	# Parsing FASTA files and detect SNPs, Produce target sequence
	if( parse_fasta() ) {
		$html_buf .= error("Parsing Fasta files failed.");
		return;
	} else { 
		$html_buf .= "<div class=\"msg_ok_string\">Fasta files parsed and database populated.</div>";
	}
	
	# Populate ecotypes table
	if( upload_ecotypes() ) {
		$html_buf .= error("Generating/Uploading of ecotypes failed.");
		return;
	} else {
		$html_buf .= "<div class=\"msg_ok_string\">Ecotype table generated.</div>";
	}
	
	# Install Fasta files into webroot
	if( install_fasta_files() ) {
		$html_buf .= error("Copying Fasta files to webroot failed.");
		return;
	} else {
		$html_buf .= "<div class=\"msg_ok_string\">Fasta files copied to webroot</div>";
	}
	
	$html_buf .= "<p><br>Data upload successfully finished. You may now proceed with Snipe4SNPs.</div>";
}

#############################################################################################################

sub error {
	my $msg = shift;
	return "<div class=\"msg_error_string\"><b>ERROR:</b> " . $msg . "</div>";
}

#############################################################################################################

sub check_msqt_schema {
	my $sth = $dbh->prepare( "select * from public.msqt_config where msqt_schema_name='$dataset_name';" );

	if (not $sth->execute()) {
		return 1;
	} 
	my $row_cnt  = $sth->rows;
	
	# if row count > 0, then a schema exists
	return ($row_cnt == 0) ? 0 : 1;
}

#############################################################################################################

sub create_msqt_schema {
	my $sql_file;
	if( $dataset_type eq "posname") {
		$sql_file = $path_to_sql_templates . "setup_msqtschema_posname.sql";
	} elsif( $dataset_type eq "chrpos" ) {
		$sql_file = $path_to_sql_templates . "setup_msqtschema_chrpos.sql";
	}
	
	open(IN, "$sql_file") or die($sql_file . " $!");
	my $sql_buffer;
	my $archive = basename($archive_name);
	($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst)=localtime(time);
	my $date = sprintf "%4d-%02d-%02d",$year+1900,$mon+1,$mday;
	while(<IN>) {
		s/YourDatasetName/$dataset_name/g;
		s/YourArchiveFile/$archive/g;
		s/PopulationDate/$date/g;
		s/YourNumberOfChromosomes/$dataset_chrom_num/g;
		
		$sql_buffer .= $_;
	}

	my $sth = $dbh->prepare( $sql_buffer );
	if (not $sth->execute()) {
		$html_buf .= "create_msqt_schema(): " . $sth->errstr;
		return 1;
	} else {
		return 0;
	}
}

#############################################################################################################

sub check_file_names {
	my $res=0;
	find sub { 
		if( -f $File::Find::name) { # skip directories and process only files
			my $f=basename $File::Find::name;
			if($f=~/[- ]/){ 
				$html_buf .= error("Bad filename found:  $File::Find::name");
				$res = 1;
			}
		}
	}, $data_path;
	return $res;
}

#############################################################################################################

sub collect_distinct_ecotypes {
	my $file_count = 0;
	find sub { 
		if( -f $File::Find::name) { # skip directories and process only files
	# 		print $File::Find::name,  "<br>";
			$file_count++;
			open(IN, $File::Find::name) or die $!;
			while(<IN>) {
				($distinct_ecotypes{$1} = "found") if m/^>(.*)$/; # Create a distinct list of ecotypes
			}
			close(IN);
		} 
	}, $data_path;
}

#############################################################################################################

sub check_for_ecot_completness {
	my $found=1;
	my $res=0;
	
	my @ecot;
	
	
	foreach my $key (sort keys %distinct_ecotypes) {
		@ecot=(@ecot, $key);
	}	
	
	my $target = $target_ecotype_name;
	
	find sub { 
		my @ecot2;
		my $buf = "";
		my $Ns;
		
		if( -f $File::Find::name) { # skip directories and process only files
			my $seq_file = $File::Find::name;
			
			# get ecotypes from sequence file
			open(FH, $seq_file) or die "$!";
			while(<FH>) {
				push(@ecot2, $1) if m/^>(.*)$/;
			}
			close(FH);
			
			if (scalar(@ecot) < scalar(@ecot2)) {
				$html_buf .= "Ecotype count mismatch." . scalar(@ecot) . "vs. " . scalar(@ecot);
				$res=1;
			}
			# get missing ecotypes
			elsif(scalar(@ecot) > scalar(@ecot2)) {
				# count bases of target
				open FH, "$seq_file" or error("$!");
				my $filebuf;
				while(<FH>) {
					$filebuf.=$_;
				}
				close(FH);
				$filebuf =~ />target\n(.*)\n/;
				my $i = length($1);
				
				if($i <= 0) {
# 					die "check_for_ecot_completness(): i=$i: Can not determine length of target-seq. Ecotype 'target' missing? Trailing space?";
					$html_buf .= error("check_for_ecot_completness(): Can not determine length of target-sequence (i=$i). Check file $seq_file. Ecotype 'target' missing? Trailing space?"); 
					$res = 1;
					return;
				}
				
				while($i--) {
					$Ns .= 'N';
				}
				for(my $i=0; $i<scalar(@ecot); $i++) {
					if( scalar(grep /^$ecot[$i]$/, @ecot2)==0 ) {
# 						$html_buf .= basename($seq_file) . ": $ecot[$i] is missing<br>";
						$buf .= ">$ecot[$i]\n" . $Ns . "\n";
					}
				}
# 				print "---\n";
			}
			
			if(length($buf) > 0) {
				
				open(FH, ">>$seq_file") or ($html_buf .= error("$!") and $res=1);
				print FH $buf;
				close(FH);
			}
			
		}
	}, $data_path;
	
	return $res;
}

#############################################################################################################

sub parse_fasta {
	my $res=0;
	if( $dataset_type eq "chrpos") {
		
		for(my $i=1; $i<=$dataset_chrom_num;$i++) {
# 			print "Processing chromosome $i<br>";
			my $dir = $data_path . "/" . $chromosome_prefix . $i;
			if (not -d $dir) {
				$html_buf .= error("Directory $dir does not exists");
				return 1;
			}
			my $file;	
			while (defined($file = <$dir/*>)) {
				if(-f $file) { # skip directories
# 					print "    -- found file $file<br>";
					$res = get_alleles("chrpos", $file, $i);
				}
			}
			
		}
		
	} elsif( $dataset_type eq "posname" ) {
			while (defined($file = <$data_path/*>)) {
				if(-f $file) { # skip directories
# 					print "    -- found file $file<br>";
					$res = get_alleles("posname", $file);
					last if($res);
				}
			}
	}
	return $res;
}

#############################################################################################################

sub get_alleles {
	my ($mode, $file, $chrom);
	$mode = shift;
	
	if($mode eq "chrpos") {
		$file = shift; 
		$chrom = shift;
	}
	elsif($mode eq "posname"){
		$file = shift;
	}
	else {
		$html_buf = "Unknown mode '$mode'. Try 'posname' or 'chrpos'.\n"; return 1;
	}
		
# 	print "in get_alleles for file $file<br>";
	
	my ($fragname, $fragid);
	my @ecot;	# The ecotype array
	my @m;		# The sequence matrix
	
	# Parse command line
	if( ($mode eq "posname") && ($file ne "")  ) {
		$fragname=basename($file);	
	}
	elsif( ($mode eq "chrpos") && ($file ne "") && ($chrom ne "")  ) {
		$fragid=basename($file);
	}
	else {
# 		print "USAGE: get_alleles( chrpos, fasta-file, chromosome )\n";
# 		print "USAGE: get_alleles( posname, fasta-file )\n";
		exit(1);
	}
	
	# Read FASTA file and generate nucleotide matrix
	open(FH, "$file") or die $!;
	while(<FH>) {
		chomp;
		if(/^>(.*)$/) { @ecot = (@ecot, $1);  }
		else { #@seq = (@seq, $_); }
			my @tmp = split //, $_;
			push @m, [ @tmp ];
		}
	}
	
	# Get count of Ecotypes
	my $cnt_ecot = scalar(@ecot);
	
	
	# Extract target sequence
	# TBD
	
	
	# Finding index of 'target'
	my $i;
	my $match_idx = -1;
	for ($i = 0; $i < @ecot; $i++) {
		if ($ecot[$i] =~ /target/) {
			$match_idx = $i;    # save the index
			last;
		}
	}
	
	if($match_idx == -1) {
		$html_buf .= error("get_alleles(): No target found! file=$file<br> Ecotype list:<br>@ecot"); 	
		return 1;
	}
	
	# Determine length of target sequence
	my $l_target = "";
	for my $aref ( $m[$match_idx] ) {
			my $len = join("", @$aref);
			$l_target = length($len)-1;
	}
	
	# Export target sequence:
	# chromosome, position, replacement, frag_id
	#print STDERR "*  Exporting target sequence for frgagment $file into $target_res_file\n";
	my $target_data_buf = "";
	
	for my $sp ( 0 .. $l_target) {
		if($mode eq "chrpos") {
			my $position = $sp+$fragid;
			$target_data_buf .= "INSERT INTO $dataset_name.target_sequences(chromosome, position, ref_base, frag_id) VALUES(";
			$target_data_buf .= "$chrom,$position,'$m[$match_idx][$sp]',$fragid );\n";
		}
		elsif($mode eq "posname"){
			my $position_name = "$fragname-$sp";
			$target_data_buf .= "INSERT INTO $dataset_name.target_sequences(position_name, ref_base, frag_id) VALUES (";
			$target_data_buf .= "'$position_name','$m[$match_idx][$sp]','$fragname' );\n";
		}
		else {
			die "Unknown mode '$mode'. Exiting.\n";
		}
	}
	
	my $sth = $dbh->prepare( $target_data_buf );
			if( not $sth->execute() ) {
				print "<div style=\"color:red; background:#F5F5F5; \"><b>ERROR: <i>" . $sth->errstr . "</i></b><br>SQL:<pre>$target_data_buf</pre></div>";
			} else {
# 				print "<div style=\"color:green; background:#F5F5F5; \"><b>Target Data uploaded for ". basename($file) . "</b></div>";
			}	
			
	
	# Analyze the matrix
	#print STDERR "*  Exporting SNP data for fragment $file into $snp_res_file\n";
	
	# Init hashes for nucleotide count and frequencies
	my %nuc_cnt = ('A',0,'C',0,'G',0,'T',0,'B',0,'D',0,'H',0,'V',0,'R',0,'Y',0,'K',0,'M',0,'S',0,'W',0,'N',0,'-',0);
	my %nuc_freq = %nuc_cnt;
	
	for my $sp ( 0 .. $l_target) {
		my $col="";
		for my $z ( 0 .. $#m) {
			if(not defined $m[$z][$sp]) { 
				$html_buf .= "get_alleles() failed: Check file $file, Ecotype $ecot[$z]"; return 1;
			}
			# Check for unallowd characters in sequence
			if($m[$z][$sp] !~ /^[ACGTBDHVRYKMSWN-]*$/) {
				$html_buf .= error("get_alleles() failed: Unallowed character in sequence. Check file $file, Ecotype $ecot[$z]"); return 1;
			}
			$col .= $m[$z][$sp];
		}
		# Check for unallowd characters in sequence
		
		# Pick out homogenous columns
		if ($col =~ /^A+$|^T+$|^G+$|^C+$|^-+$|^B+$|^D+$|^H+$|^V+$|^R+$|^Y+$|^K+$|^M+$|^S+$|^W+$/) { 
			#print "  Only AATGC in $sp: '$col'\n" if($DEBUG); 
		}
		# Pick out columns containing only Ns and one other base like (GGGNGGG) 
		elsif ($col =~ /^[AN]+$|^[TN]+$|^[GN]+$|^[CN]+$|^[-N]+$|^[BN]+$|^[DN]+$|^[HN]+$|^[VN]+$|^[RN]+$|^[YN]+$|^[KN]+$|^[MN]+$|^[SN]+$|^[WN]+$/) { 
			#print "  Homogenous + N in $sp: '$col'\n" if($DEBUG); 
		}
		else {
			# SNP candidate!
			#print "SNP at $sp: $col\n" if($DEBUG);
			
			# Count the occurance of each nucletide
			foreach my $key (sort keys %nuc_cnt) {
				$nuc_cnt{$key} = grep(/$key/, split(//,$col));
			}
	
			# Compute allele frequency for each nocletide
			foreach my $key (sort keys %nuc_freq) {
				$nuc_freq{$key} = 1 / $cnt_ecot * $nuc_cnt{$key} if($nuc_cnt{$key}!=0);
			}
	
			# Print output:
			# chromosome, position, ecotype, replacement, frag_id, allele_freq
			my $snp_data_buf = "";
			
			for my $_z ( 0 .. $#m ) {
				
				if($mode eq "chrpos") {
					my $position = $sp+$fragid;
					$snp_data_buf .= "INSERT INTO $dataset_name.snps(chromosome, position, ecotype, replacement, frag_id, allele_freq) VALUES(";
					$snp_data_buf .= "$chrom,$position,'$ecot[$_z]','$m[$_z][$sp]',$fragid,";
				}
				elsif($mode eq "posname"){
					my $position = $sp;
					$snp_data_buf .= "INSERT INTO $dataset_name.snps(position_name, ecotype, replacement, frag_id, allele_freq) VALUES (";
					$snp_data_buf .= "'$fragname-$position','$ecot[$_z]','$m[$_z][$sp]','$fragname',";
				}
				else {
					die "Unknown mode '$mode'. Exiting.\n";
				}
				foreach my $key (sort keys %nuc_freq) {
# 					$html_buf .= error("DEBUG: get_alleles(): checking key $key => $nuc_freq{$key}");
					if( $m[$_z][$sp] eq "$key") {
						$snp_data_buf .= sprintf("%.2f", $nuc_freq{$key}) . ");"; 
					}
					else {
# 						die "Can not compute allele_freq";
# 						$html_buf .= error("get_alleles() failed: Can not compute allele_freq for column = '$col' <br> key = '$key'<br>  _z = '$#m'<br>  l_target = '$l_target'<br>  sp = '$sp'");
# 						$html_buf .= error("DEBUG: get_alleles(): $m[$_z][$sp] != $key");
# 						return 1;
					}
				}
				
			}
			
			
			my $sth = $dbh->prepare( $snp_data_buf );
			if( not $sth->execute() ) {
				print STDERR "sth->execute failed for $snp_data_buf\n";
				$html_buf .= error("get_alleles() failed: $sth->errstr");
				return 1;
				
			} else {
# 				print "<div style=\"color:green; background:#F5F5F5; \"><b>SNP Data uploaded for ". basename($file) . "</b></div>";
			}
				
			
		}
	}
}

#############################################################################################################

sub upload_ecotypes {

	my $ecotype_sql = "INSERT INTO $dataset_name.ecotypes (ecotype) SELECT DISTINCT $dataset_name.snps.ecotype FROM $dataset_name.snps;";
			
	my $sth = $dbh->prepare( $ecotype_sql );
	if( not $sth->execute() ) {
		$html_buf .= error($sth->errstr);
		return 1;	
	}
	
	return 0;
}

#############################################################################################################

sub install_fasta_files {	# copies fasta files into webroot
	$cmd_mkdir = "mkdir -p $fasta_files_dir/$dataset_name";
	system( $cmd_mkdir ) == 0 or ($html_buf .= error("system $cmd_mkdir failed: $?") and return 1);

	my $from = "$data_path/*";
	my $to = "$fasta_files_dir/$dataset_name/";
	$cmd_mv = "mv  $from $to";
	system( $cmd_mv ) == 0 or ($html_buf .= error("system $cmd_mv failed: $?") and return 1);
	
	return 0;
}

#############################################################################################################

sub show_documentation {
	$html_buf = <<EOT;
		
	<h1>Documentation</h1>
	
	<div class="doc_container">
	TBD
	</div>
EOT
}

#############################################################################################################

sub show_about {
	$html_buf = <<EOT;
		
	<h1>About</h1>
	<div class="doc_container">
	
	MSQT/Admin Version $VERSION
	<p>
	Copyright &copy; 2007 Max Planck Institute for Developmental Biology,<br>
 	http://www.eb.tuebingen.mpg.de
	<br><br>
	Written by Joffrey Fitz.
	<br><br>
	This program is part of the MSQT project by Norman Warthmann, Joffrey Fitz <br>
	and Detlef Weigel (http://msqt.weigelworld.org)<br>
	<br>
	This program is free software; you can redistribute it and/or modify it <br>                     
	under the terms of the GNU General Public License as published by the Free <br>                    
	Software Foundation; either version 2, or (at your option) any later <br>
	version. <br>
	<br>
	This program is distributed in the hope that it will be useful, but <br>
	WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY <br> 
	or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License <br>
	for more details. <br>

	<p>
	
	MSQT/Admin is part of the MSQT Project by Norman Warthmann, Joffrey Fitz and Detlef Weigel.
	<p>
	
	This project is hosted at <a href="http://msqt.weigelworld.org">msqt.weigelworld.org</a>.
	<p>
	
	</div>
EOT
}

#############################################################################################################

sub show_snipe4snps {
	#
	# Assembly sniped information
	#
	my $query_datasets =<<EOQ;
SELECT msqt_schema_name FROM public.msqt_config
EOQ

	my $sth = $dbh->prepare( $query_datasets );
	$sth->execute or die;
	my @datasets;
	my $rows;
	
	while ( $row = $sth->fetchrow ) {
		push(@datasets, $row);
	}

	my $query_tmpl =<<EOQ;
SELECT msqt_schema_name AS "Dataset name", msqt_schema_type AS "Type",
(SELECT count(*) FROM dataset.sniped) AS "Sniped SNPs",
s4s_diff_threshold AS "DT",
s4s_left_neigbh_threshold AS "LnT", s4s_right_neigbh_threshold AS "RnT"
FROM public.msqt_config
WHERE msqt_schema_name='dataset'
EOQ

	my $query="";
	for(my $i=0; $i<scalar(@datasets); $i++) {
		$query .= $query_tmpl;
		$query =~ s/dataset/$datasets[$i]/g;
		$query .= " UNION " if($i<scalar(@datasets)-1);
	}
	
	print "show_snipe4snps:<br>$query" if($DEBUG);
	
	#
	# Define HTMLTableWidget
	#
	my $table4 = new SQLQuery2HTMLTable;
	
	my $tablestyle		  = "width:100%; border-collapse:normal; ";
	my $headercellstyle	  = "background:#D3D4DA; text-align:left; white-space:nowrap;";
	my $evenrowstyle      = "background-color: #E4EAEF; color:black; vertical-align:top";
	my $oddrowstyle       = "background-color: #F5F5F5; color:black; vertical-align:top";
	my $evenrowstylehover = "background-color: #77B1E2; color:black; vertical-align:top";
	my $oddrowstylehover  = "background-color: #77B1E2; color:black; vertical-align:top";
	
	# 	my $front_extension_content = "<a href=\"$script_loc?viewfasta=__COL0__\" target=\"__aaces_fasta__COL0____COL1__\">View FASTA</a>";
	my $back_extension_content 	= "<div style=\"white-space:nowrap\"><a href=\"$script_loc?tab=Snipe4SNPs&action=s4s_empty_confirm&schema=__COL0__\">empty</a>"; # __COLn__ will be substitued to the value of colmun n in row k
	$back_extension_content .= "&nbsp;&nbsp;|&nbsp;&nbsp;<a href=\"$script_loc?action=show_snipe4snps_parameters&tab=Snipe4SNPs&schema=__COL0__&schema_type=__COL1__\">compile SNPs</a></div>";
	
	# 	my $custom_header_template = "<a href=\"$script_loc?search_string=$search_string&search_field=$search_field&row_limit=$row_limit&orderquery=__HEADER_TITLE__&search=Hit+me\">__HEADER_TITLE__</a>";
	
	
# 	my $expr = '$col_content =~ s/(.*)(\.[zy][ab].exp)/<div style="white-space:nowrap"><a href="?viewallreads=$1" target="__aaces_allreads_$1" title="View all available reads for $1">-><-<\/a>&nbsp;&nbsp;$1$2<\/div>/; ';
# 	my $expr_snpcount = '$col_content =~ s/0/none/;';
	my $expr_snpcount = '$col_content =~ s/^0$/-/';
	my %custom_cells;
	$custom_cells{2} = "$expr_snpcount";
	
	
	$table4->set( 
		dbh => $dbh, query => $query,
		tablestyle => $tablestyle, headercellstyle => $headercellstyle,
		twocolorrows => 1, 
		evenrowstyle => $evenrowstyle, oddrowstyle => $oddrowstyle,
		rowhighlight => 1,
		evenrowstylehover => $evenrowstylehover, oddrowstylehover => $oddrowstylehover,
		back_extension_column => "Action", back_extension_content => $back_extension_content,
		# 		front_extension_column => "Sequence file", front_extension_content => $front_extension_content,
		custom_header => 0,
		# 		custom_header_template => $custom_header_template,
		expr => \%custom_cells
	);
	
	#
	# 	Assembly HTML
	#
	
	
	$html_buf = print_headline("Snipe4SNPs");
	$html_buf .= $table4->gen_html_table;
	$html_buf .= "<p><a href=\"$script_loc?tab=Snipe4SNPs\"><b>Refresh list</b></a>\n";
}

#############################################################################################################

sub show_snipe4snps_parameters {
	my ($schema, $schema_type) = @_;
# javascript:openWindow('$script_loc?action=run_snipe4snps&tab=Snipe4SNPs&schema=__COL0__', 'Snipe4SNPs Processing Monitor', 600, 500);

# $html_buf = <<EOF;
# <script language="JavaScript">
# <!--
# function openWindow(url, name, width, height) {
#   mywin = window.open(url, name, 'width=' + width + ',height=' + height + ',toolbar=0,location=0,directories=0,status=0,menubar=0,scrollbars=0,resizable=1')
# }
# //  -->
# </script
# EOF
# $div_threshold, $right_neigbh_threshold, $left_neigbh_threshold

	$html_buf = print_headline("Snipe4SNPs <i>Step 1 of 2:</i>");
	$html_buf .= <<EOF;
	
	<div id="code">
	<h2 class="title">Parameters:</h2>
	
	Adjust these parameters only if you know what you are doing.<br>
	The default values are a good choice.<br><br>
	
	<FORM ACTION="$script_loc" METHOD="post" ENCTYPE="multipart/form-data">
	
	<table>
	<tr>
	<td>
		diff_threshold:
	</td>
	<td>
		<input type="text" name="div_threshold" id="div_threshold" size="10" value="1.1">
	</td>
	</tr>
	
	<tr>
	<td>
		right_neigbh_threshold:
	</td>
	<td>
		<input type="text" name="right_neigbh_threshold" id="right_neigbh_threshold" size="10" value="1">
	</td>
	</tr>
	
	<tr>
	<td>
		left_neigbh_threshold:
	</td>
	<td>
		<input type="text" name="left_neigbh_threshold" id="left_neigbh_threshold" size="10" value="1">
	</td>
	</tr>
	
	</table>
	
	<p><br><br>
	
	<input type="submit" name="submit_button" value="OK">
	<input type="hidden" name="action" value="confirm_snip4snps_parameter">
	<input type="hidden" name="tab" value="Snipe4SNPs">
	<input type="hidden" name="schema" value="$schema">
	<input type="hidden" name="schema_type" value="$schema_type">
	
	</FORM>
	<br>
	
	</div>
EOF
}

#############################################################################################################

sub show_snipe4snps_confirm {
	my ($schema, $schema_type, $div_threshold, $right_neigbh_threshold, $left_neigbh_threshold) = @_;
	$div_threshold = "<b style=\"color: red\">undefined. There Must be something worng</b>" if($div_threshold eq "");
	$right_neigbh_threshold = "<b style=\"color: red\">undefined. There Must be something worng</b>" if($right_neigbh_threshold eq "");
	$left_neigbh_threshold = "<b style=\"color: red\">undefined. There Must be something worng</b>" if($left_neigbh_threshold eq "");

$html_buf = <<EOF;
<script language="JavaScript">
<!--
function openWindow(url, name, width, height) {
  mywin = window.open(url, name, 'width=' + width + ',height=' + height + ',toolbar=0,location=0,directories=0,status=0,menubar=0,scrollbars=0,resizable=1')
}
//  -->
</script
EOF

	$html_buf .= print_headline("Snipe4SNPs <i>Step 2 of 2: Start Snipe4SNPs</i>");
	$html_buf .= <<EOF;
	
	<div id="code">
	<h2 class="title">Confirm Parameters:</h2>
	
	Please check your parameters and start Snipe4SNPs with the link below.<br><br><br>
	<table>
	<tr>
	<td class="server_info_tags">
		Dataset:
	</td>
	<td class="server_info_values">
		$schema
	</td>
	</tr>
	
	<td class="server_info_tags">
		Type:
	</td>
	<td class="server_info_values">
		$schema_type
	</td>
	</tr>
	
	<tr>
	<td class="server_info_tags">
		diff_threshold:
	</td>
	<td class="server_info_values">
		$div_threshold
	</td>
	</tr>
	
	<tr>
	<td class="server_info_tags">
		right_neigbh_threshold:
	</td>
	<td class="server_info_values">
		$right_neigbh_threshold
	</td>
	</tr>
	
	<tr>
	<td class="server_info_tags">
		left_neigbh_threshold:
	</td>
	<td class="server_info_values">
		$left_neigbh_threshold
	</td>
	</tr>
	
	</table>
	
	<p><br><br>
	<a href="javascript:openWindow('$script_loc?action=run_snipe4snps&tab=Snipe4SNPs&schema=$schema&schema_type=$schema_type&div_threshold=$div_threshold&left_neigbh_threshold=$left_neigbh_threshold&right_neigbh_threshold=$right_neigbh_threshold', 'Snipe4SNPs Processing Monitor', 600, 500);">Click here to start Snipe4SNPs</a>. 
	(This will open a new Window)
	
	</div>
EOF
}

#############################################################################################################

sub run_snipe4snps {
	my ($schema, $schema_type, $div_threshold, $right_neigbh_threshold, $left_neigbh_threshold) = @_;
	
	print "run_snipe4snps(): Got params: ($schema, $schema_type, $div_threshold, $right_neigbh_threshold, $left_neigbh_threshold)\n" if ($DEBUG);
	print "left_neigbh_threshold=$left_neigbh_threshold\n" if($DEBUG);
	
	my $query =<<EOQ;

	UPDATE public.msqt_config SET 
		s4s_diff_threshold='$div_threshold', 
		s4s_left_neigbh_threshold='$right_neigbh_threshold', 
		s4s_right_neigbh_threshold='$left_neigbh_threshold'
	WHERE msqt_schema_name = '$schema'
EOQ
	
          if($DEBUG) { print "query: $query<br>"; }

	my $sth = $dbh->prepare( $query );
	if( not $sth->execute() ) {
		$html_buf .= "<div style=\"color:red; background:#F5F5F5; border:solid 1px grey;\"><b>ERROR: <i>" . $sth->errstr . "</i></b></div><p>";
		return;
	} else {
	
	$html_buf = print_headline("Snipe4SNPs Processing Monitor");
	$html_buf .= <<EOF;
	<div class="warning" id="warning">Snipe4SNPs is running. <br>Please do not close or reload this window.<br><br><p>

	<iframe src="$sniped_stream_cgi?schema=$schema&schema_type=$schema_type&div_threshold=$div_threshold&left_neigbh_threshold=$left_neigbh_threshold&right_neigbh_threshold=$right_neigbh_threshold" width="90%"  name="snipe4snpsmonitor">
	
	<p>Ihr Browser kann leider keine eingebetteten Frames anzeigen:
	Sie k&ouml;nnen die eingebettete Seite &uuml;ber den folgenden Verweis
	aufrufen: <a href="../../../index.htm">SELFHTML</a></p>
	
	</iframe>
</div>
EOF

	}
}

#############################################################################################################

sub s4s_empty_confirm {
	my $schema = shift;
	
	$html_buf = print_headline("Snipe4SNPs");
	
	$html_buf .= <<EOS;
<div class="error_box">
<div class="error_box_headline">Confirmation Request</div><br>
Do you really want to delete sniped SNPs for schema <i>$schema</i>?<p>

 
<a href="$script_loc?action=s4s_empty&schema=$schema&tab=Snipe4SNPs"><b>Yes</b></a>
<a href="$script_loc?tab=Snipe4SNPs">No</a>

</div>
EOS
}

#############################################################################################################

sub s4s_empty {
	my $schema = shift;
	
	$html_buf = print_headline("Snipe4SNPs");
	
	my $myerr = "";
	
	my $query =<<EOQ;
	-- delete sniped snips
	DELETE FROM $schema.sniped;
	
	-- Delete SNIPED config parameters from config table
	UPDATE public.msqt_config SET 
		s4s_diff_threshold='', 
		s4s_left_neigbh_threshold='', 
		s4s_right_neigbh_threshold=''
	WHERE msqt_schema_name = '$schema'
EOQ
	
  	if($DEBUG) { print "query: $query<br>"; }

	my $sth = $dbh->prepare( $query );
	if( not $sth->execute() ) {
		$html_buf .= "<div style=\"color:red; background:#F5F5F5; border:solid 1px grey;\"><b>ERROR: <i>" . $sth->errstr . "</i></b></div><p>";
		
	} else {
# 		$html_buf .= <<EOS;
# <div style="padding:20px; background:#F5F5F5; border:solid 1px grey;">
# <div style="color:green; font-size:biggest; font-weight:bold;">Confirmation Request</div><br>
# SNPs for schema <i>$schema</i> successfully deleted.<p>
# 
#  
# <a href="$script_loc?tab=Snipe4SNPs"><b>Return to main page</b></a>
# 
# </div>
# EOS
		return;
	}
}
