#!/usr/bin/perl
# written by norman warthmann and stephan ossowski, aug 22nd-26th 2005
# heavy modified and extended by Joffrey Fitz

use strict;
use warnings;
use DBI;
use Switch;

package distinct_snps_chrpos;

sub new {
	my $self = {
		chromosome => '',
		start => '',
		end => '',
		dbtable => '',
		dbh => '',
		gen_out => '',
		ecotypes => {},
		distinct_snp_hash => {},
		all_snp_hash => {},
		target_snp_hash => {},
		all_frag_hash => {},
		output => [],
	};
	bless $self;
	return $self;
}

sub init {
	my ($self, $chromosome, $start, $end, $dbh, $dbtable, %ecotypes) = @_;
	$self->{chromosome} = $chromosome;
	$self->{start} = $start;
	$self->{end} = $end;
	%{$self->{ecotypes}} = %ecotypes;
	$self->{dbtable} = $dbtable;

	my $ecotype_list_one = '';
	my $ecotype_list_two = '';
	my $ecotype_list_three = '';
	my %ecotype_hash_one = ();
	my %ecotype_hash_two = ();
	my $current_position = -1;
	my @A = (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0);
	my @B = (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0);
	my @C = (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0);
	my @D = (0,0,0,0,0);
	my $buf = '';

	# Prepare two lists of ecotypes for SQL statement
	foreach my $ecotype (keys %ecotypes){
		if($ecotypes{$ecotype} == 0) {
			$ecotype_list_one = $ecotype_list_one . " (ecotype='$ecotype') ";
			$ecotype_hash_one{$ecotype} = 1;
		}
		elsif($ecotypes{$ecotype} == 1) {
			$ecotype_list_two = $ecotype_list_two . " (ecotype='$ecotype') ";
			$ecotype_hash_two{$ecotype} = 1;
		}
		elsif($ecotypes{$ecotype} == 2) {
			$ecotype_list_three = $ecotype_list_three . " (ecotype='$ecotype') ";
			#$ecotype_hash_three{$ecotype} = 1;
		}
	}
	$ecotype_list_one=~s/\)  \(/\) or \(/g;
	$ecotype_list_two=~s/\)  \(/\) or \(/g;
	$ecotype_list_three=~s/\)  \(/\) or \(/g;

	# speedup the query if both positions are 0,
	# in other words do not look at ranges, look at the whole chromosome
	my $silencer='';
	if(($start==0) and ($end==0)) {
		$silencer = "--";
	}
	# Select all snp for range and ecotypes
	my $q = 	"SELECT position, ecotype, replacement, frag_id
			 FROM 	$dbtable.snps
			 WHERE 	chromosome = $chromosome AND
				$silencer position BETWEEN $start AND $end AND
				($ecotype_list_one OR	-- ecotype_list_one 
				 $ecotype_list_two OR	-- ecotype_list_two
				 $ecotype_list_three)	-- ecotype_list_three
			 ORDER BY position
			";
	#print "<pre>$q</pre>";
	my $q_sth = $dbh->prepare($q);
	$q_sth->execute();


	# Loop through results
	my $frag_id;
	while(my $q_ref = $q_sth->fetchrow_hashref()) {
		my $position = $q_ref->{'position'};
		my $ecotype = $q_ref->{'ecotype'};
		my $replacement = $q_ref->{'replacement'};
		$frag_id = $q_ref->{'frag_id'};

		# store frag_id
		$self->{all_frag_hash}{$position} = $frag_id;

		if($position > $current_position) {
			if($current_position != -1) {
				my $A_as_number = $A[0]*1+$A[1]*10+$A[2]*100+$A[3]*1000+$A[4]*10000;
				my $B_as_number = $B[0]*1+$B[1]*10+$B[2]*100+$B[3]*1000+$B[4]*10000;
				my $C_as_number = $C[0]+$C[1]+$C[2]+$C[3]+$C[4];

				if($C_as_number > 1) {
					$self->{all_snp_hash}{$current_position} = &bv2_bases(@C);
				}
				elsif(	($C[0] != $D[0]) || ($C[1] != $D[1]) || ($C[2] != $D[2]) ||
					($C[3] != $D[3]) || ($C[4] != $D[4]) ) {
					$self->{target_snp_hash}{$current_position} = &bv2_bases(@C);
				}
				if(!(
					($A_as_number==0) ||
					($B_as_number==0) ||
					($A_as_number - $B_as_number == 0) ||
					($A[0]==1 && $B[0]==1) ||
					($A[1]==1 && $B[1]==1) ||
					($A[2]==1 && $B[2]==1) ||
					($A[3]==1 && $B[3]==1) ||
					($A[4]==1 && $B[4]==1)
				)) {
					$buf .= "$current_position\t@A\t@B\t";
					$buf .= &bv2_bases(@A); $buf .= "\t";
					$buf .= &bv2_bases(@B); $buf .= "\n";
					$self->{distinct_snp_hash}{$current_position} = &bv2_bases(@A) . "/" . &bv2_bases(@B);
				}
			}

			$current_position = $position;
			@A = (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0);
			@B = (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0);
			@C = (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0);
			@D = (0,0,0,0,0);
		}

		#check if different from target
		if($ecotype eq "target") {
			switch ($replacement) {
				case "A"        {$D[0]=1}
				case "T"        {$D[1]=1}
				case "G"        {$D[2]=1}
				case "C"        {$D[3]=1}
				case "-"        {$D[4]=1}
			}
		}

		#binary coding of replacement for all hash
		else{
			switch ($replacement) {
				case "A"        {$C[0]=1}
				case "T"        {$C[1]=1}
				case "G"        {$C[2]=1}
				case "C"        {$C[3]=1}
				case "-"        {$C[4]=1}
				case "N"        {$C[5]=1}
				case "R" 		{$C[6]=1}
				case "Y" 		{$C[7]=1}
				case "K" 		{$C[8]=1}
				case "M" 		{$C[9]=1}
				case "S" 		{$C[10]=1}
				case "W" 		{$C[11]=1}
				case "B" 		{$C[12]=1}
				case "D" 		{$C[13]=1}
				case "H" 		{$C[14]=1}
				case "V" 		{$C[15]=1}
				else {print "First ecotype group has problem at\t$position\n"}
			}
		}

		if(exists $ecotype_hash_one{$ecotype}) {
			#binary coding of replacements for distinct snps
			switch ($replacement) {
				case "A" 	{$A[0]=1}
				case "T" 	{$A[1]=1}
				case "G" 	{$A[2]=1}
				case "C" 	{$A[3]=1}
				case "-" 	{$A[4]=1}
				case "N" 	{$A[5]=1}
				case "R" 	{$A[6]=1}
				case "Y" 	{$A[7]=1}
				case "K" 	{$A[8]=1}
				case "M" 	{$A[9]=1}
				case "S" 	{$A[10]=1}
				case "W" 	{$A[11]=1}
				case "B" 	{$A[12]=1}
				case "D" 	{$A[13]=1}
				case "H" 	{$A[14]=1}
				case "V" 	{$A[15]=1}
				else {print "First ecotype group has problem at\t$position\n"}
			}
		}
		elsif(exists $ecotype_hash_two{$ecotype}) {
			#binary coding of replacements for distinct snps
			switch ($replacement) {
				case "A"        {$B[0]=1}
				case "T"        {$B[1]=1}
				case "G"        {$B[2]=1}
				case "C"        {$B[3]=1}
				case "-"        {$B[4]=1}
				case "N"        {$B[5]=1}
				case "R" 		{$B[6]=1}
				case "Y" 		{$B[7]=1}
				case "K" 		{$B[8]=1}
				case "M" 		{$B[9]=1}
				case "S" 		{$B[10]=1}
				case "W" 		{$B[11]=1}
				case "B" 		{$B[12]=1}
				case "D" 		{$B[13]=1}
				case "H" 		{$B[14]=1}
				case "V" 		{$B[15]=1}
				else {print "Second ecotype group has problem at\t$position\n"}
			}
		}
	}

	# Postprocessing last entry from db
	my $A_as_number = $A[0]*1+$A[1]*10+$A[2]*100+$A[3]*1000+$A[4]*10000;
	my $B_as_number = $B[0]*1+$B[1]*10+$B[2]*100+$B[3]*1000+$B[4]*10000;
	my $C_as_number = $C[0]+$C[1]+$C[2]+$C[3]+$C[4];

	if($C_as_number > 1) {
		$self->{all_snp_hash}{$current_position} = &bv2_bases(@C);
		$self->{all_frag_hash}{$current_position} = $frag_id;
	}
	elsif(  ($C[0] != $D[0]) || ($C[1] != $D[1]) || ($C[2] != $D[2]) ||
		($C[3] != $D[3]) || ($C[4] != $D[4]) ) {
		$self->{target_snp_hash}{$current_position} = &bv2_bases(@C);
	}
	if(!(
		($A_as_number==0) || ($B_as_number==0) || ($A_as_number - $B_as_number == 0) ||
		($A[0]==1 && $B[0]==1) || ($A[1]==1 && $B[1]==1) || ($A[2]==1 && $B[2]==1) ||
		($A[3]==1 && $B[3]==1) || ($A[4]==1 && $B[4]==1)
	)) {
		$buf .= "$current_position\t@A\t@B\t"; $buf .= &bv2_bases(@A);
		$buf .= "\t"; $buf .= &bv2_bases(@B); $buf .= "\n";
		$self->{distinct_snp_hash}{$current_position} = &bv2_bases(@A) . "/" . &bv2_bases(@B);
	}

	# Create clustered output array
	$self->cluster($chromosome, $buf, $dbtable, $dbh);
}


sub cluster {
	my ($self, $chromosome, $buf, $dbtable, $dbh) = @_;

	my @positionsarray = ();
	my @seq1array = ();
	my @seq2array = ();
	my $start;

	my @lines = split("\n",$buf);

	foreach my $line (@lines) {
		chomp($line);
		my ($position, $bin1, $bin2, $base1, $base2) = split("\t",$line);
		push(@positionsarray, $position);
		push(@seq1array,"$base1");
		push(@seq2array,"$base2");
	}

	my $flag=0;
	my @indel_allele1=();
	my @indel_allele2=();
	my $tmp = '';

	push(@positionsarray, 0);


	# if use primers
	my $primer_q = "select distinct frag_id from $dbtable.primers;";
        my $primer_q_sth = $dbh->prepare($primer_q);
	$primer_q_sth->execute() || die;
	my @primer_frag_ids;
	while(my $primer_q_ref = $primer_q_sth->fetchrow_hashref()) {
		push(@primer_frag_ids, $primer_q_ref->{'frag_id'});
	}
	$primer_q_sth->finish() || die;
	
	#print STDERR "distinct_snps_chrpos:primers found:@primer_frag_ids";
	#if( scalar( grep(/1149096/, @primer_frag_ids) ) ) {
	#	print STDERR "distinct_snps_chrpos:primer found";
	#}
	# end primer stuff


	for (my $i=1; $i < scalar(@positionsarray); $i++) {
		if (($flag==0) && ($positionsarray[$i] != $positionsarray[$i-1]+1)) {
			$tmp = $chromosome ."\tSNP\t". $positionsarray[$i-1] ."\t". $seq1array[$i-1] ."\t". 
			$seq2array[$i-1];
			# frag_id
			# highlighted if primer exists
			if( scalar( grep(/$self->{all_frag_hash}{$positionsarray[$i-1]}/, @primer_frag_ids) ) ) {
				$tmp .= "\t<font style=\"background-color:rgb(255, 186, 66);\">"
				. $self->{all_frag_hash}{$positionsarray[$i-1]}
				. "</font>\t";
			}
			# no primer, print frag_id
			else {
				$tmp .= "\t" . $self->{all_frag_hash}{$positionsarray[$i-1]} . "\t";
			}
			
			# link definition for jalview
			$tmp .= "<a href=\"javascript: openWindow("
			#. "'http://merkur.eb.local/msqt/cgi-bin/msqt-jalview.cgi?jalview=jalview&chr=$chromosome&frag_id="
			. "'./msqt-jalview.cgi?jalview=jalview&dbtable=$dbtable&chr=$chromosome&frag_id="
			. $self->{all_frag_hash}{$positionsarray[$i-1]}
			. "','"
			. $self->{all_frag_hash}{$positionsarray[$i-1]}
			."','450','200')\""
			. "\">"
			. "alignment"
			. "</a>"
			. "\t"
			# link definition for gf
			#. "<a href=\"http://merkur.eb.local/msqt/cgi-bin/msqt-adf.cgi"
			. "<a href=\"./msqt-adf.cgi"
			. "?sbe_database=<database><ecot_list_1><ecot_list_2><ecot_list_3>&position=$positionsarray[$i-1]&chromosome=$chromosome&compute_snps=Compute+SNPs\">"
			. "assay_development_format</a>";
			push(@{$self->{output}}, $tmp);

		}
		elsif (($flag==0) && ($positionsarray[$i] == $positionsarray[$i-1]+1)) {
			$flag=1;
			$tmp = $chromosome ."\tindel\t". $positionsarray[$i-1];
			push(@indel_allele1,"$seq1array[$i-1] ");
			push(@indel_allele2,"$seq2array[$i-1] ");
		}
		elsif (($flag==1) && ($positionsarray[$i] == $positionsarray[$i-1]+1)) {
			push(@indel_allele1,"$seq1array[$i-1] ");
			push(@indel_allele2,"$seq2array[$i-1] ");
		}
		elsif (($flag==1) && ($positionsarray[$i] != $positionsarray[$i-1]+1)) {
			$flag=0;
			push(@indel_allele1,"$seq1array[$i-1] ");
			push(@indel_allele2,"$seq2array[$i-1] ");
			$tmp .= "\t" . join ("",@indel_allele1) ."\t". join ("",@indel_allele2);
				
			# frag_id
			# highlighted if primer exists
			if( scalar( grep(/$self->{all_frag_hash}{$positionsarray[$i-1]}/, @primer_frag_ids) ) ) {
				$tmp .= "\t<font style=\"background-color:rgb(255, 186, 66);\">"
				. $self->{all_frag_hash}{$positionsarray[$i-1]}
				. "</font>\t";
			}
			# no primer, print frag_id
			else {
				$tmp .= "\t" . $self->{all_frag_hash}{$positionsarray[$i-1]} . "\t";
			}
				
				# link definition for jalview
				$tmp .= "<a href=\"javascript: openWindow("
				#. "'http://merkur.eb.local/msqt/cgi-bin/msqt-jalview.cgi?jalview=jalview&chr=$chromosome&frag_id="
				. "'./msqt-jalview.cgi?jalview=jalview&dbtable=$dbtable&chr=$chromosome&frag_id="
				. $self->{all_frag_hash}{$positionsarray[$i-1]}
				. "','"
				. $self->{all_frag_hash}{$positionsarray[$i-1]}
				."','450','200')\""
				. "\">"
				. "alignment"
				. "</a>"
				. "\t"
				# link definition for gf
				#. "<a href=\"http://merkur.eb.local/msqt/cgi-bin/msqt-adf.cgi"
				. "<a href=\"./msqt-adf.cgi"
				. "?sbe_database=<database><ecot_list_1><ecot_list_2><ecot_list_3>&start_pos=$positionsarray[$i-1]&chromosome=$chromosome&compute_snps=Compute+SNPs\">"
				. "assay_development_format</a>";
			push(@{$self->{output}}, $tmp);
			@indel_allele1=();
			@indel_allele2=();
		}
		else { print  $chromosome ."\tthere is somthing wrong!\n"; }
	}
}


#####################################################
### backtranslates the binary code into bases
#####################################################
sub bv2_bases
{
	my @bases = ('A','T','G','C','-','N','R','Y','K','M','S','W','B','D','H','V');
	my $buf;
	my @bv = @_;

	# Uebersetze Bitvector in Basen
	for(my $i=0; $i<scalar(@bv); $i++) {
		$buf.= "$bases[$i]," if $bv[$i];
	}
	chop($buf);
	return $buf;
}


sub find_frag {
	# Get fragment of distinct snp
	my ($self, $chromosome, $position, $dbh, $dbtable, %ecotypes) = @_;
	my $q_frag = "SELECT frag_id FROM " . $dbtable . ".target_sequences
			WHERE   chromosome = $chromosome AND position = $position";

#print "<pre><b>distinct_snps_chrpos: find_frag(): q_frag:</b><br>$q_frag</pre>";

	my $sth_frag = $dbh->prepare($q_frag);
	$sth_frag->execute();
	my $ref_frag = $sth_frag->fetchrow_hashref();
	my $frag_id = $ref_frag->{'frag_id'};

	$q_frag =    	"SELECT min(position) AS minpos, max(position) AS maxpos
			FROM " . $dbtable . ".target_sequences
			WHERE 	frag_id = $frag_id
			";

#print "<pre><b>distinct_snps_chrpos: find_frag(): q_frag2:</b><br>$q_frag</pre>";

	$sth_frag = $dbh->prepare($q_frag);
	$sth_frag->execute();
	$ref_frag = $sth_frag->fetchrow_hashref();
	my $start = $ref_frag->{'minpos'};
	my $end   = $ref_frag->{'maxpos'};
	$self->init($chromosome, $start, $end, $dbh, $dbtable, %ecotypes);
}

sub genaissance
{
	my ($self, $dbh) = @_;

	# Get target sequence of fragment
	my $q_target = "SELECT position, ref_base FROM target_sequence_table
			WHERE   chromosome = " . $self->{chromosome} . " AND
			position BETWEEN " . $self->{start} . " AND " . $self->{end} . "
			ORDER BY position
			";
	my $sth_target = $dbh->prepare($q_target);
	$sth_target->execute();
	while(my $ref_target = $sth_target->fetchrow_hashref()) {
		my $position = $ref_target->{'position'};
		my $base = $ref_target->{'ref_base'};
		if(exists $self->{distinct_snp_hash}{$position}) {
			$self->{gen_out} = $self->{gen_out} . "[" . $self->{distinct_snp_hash}{$position} . "]";
		}
		elsif(exists $self->{all_snp_hash}{$position}) {
			$self->{gen_out} = $self->{gen_out} . "{" . $self->{all_snp_hash}{$position} . "}";
		}
		elsif(exists $self->{target_snp_hash}{$position}) {
			$self->{gen_out} = $self->{gen_out} . "{" . $self->{target_snp_hash}{$position} . "}";
		}
		else{
			$self->{gen_out} .= $base;
		}
	}
}


sub genaissance_html
{
	my ($self, $dbh) = @_;
	my $primary_snp_pos;

	# Get target sequence of whole FRAGMET
	my $q_target = "SELECT position, ref_base FROM "
			. $self->{dbtable} . ".target_sequences "
			. "WHERE   chromosome = " . $self->{chromosome} . " AND "
			. "position BETWEEN " . $self->{start} . " AND " . $self->{end} . " "
			. "ORDER BY position";
	
#print "<pre><b>distinct_snps_chrpos.pm: genaissance_html(): q_target:</b><br> $q_target</pre>";

	my $sth_target = $dbh->prepare($q_target);
	$sth_target->execute();

	my @pos_arr;
	my %target;
	while(my $ref_target = $sth_target->fetchrow_hashref()) {
		$target{ $ref_target->{'position'} } = $ref_target->{'ref_base'};
	}

	my $i=0;
	foreach my $position (sort {$a <=> $b} keys %target) {
		#my $value = $target{$position};
		if(exists $self->{distinct_snp_hash}{$position}) {
			$i++;
			$self->{gen_out} = $self->{gen_out} . "<$i>:";
			foreach my $position2 (sort {$a <=> $b} keys %target) {
				my $value = $target{$position2};

				if($position2==$position) { # this is the primary snp
					$primary_snp_pos = $position;
					$self->{gen_out} = $self->{gen_out} . "[" . $self->{distinct_snp_hash}{$position2} . "]";
				}
				elsif(exists $self->{all_snp_hash}{$position2}) {
					$self->{gen_out} = $self->{gen_out} . "{" . $self->{all_snp_hash}{$position2} . "}";
				}
				elsif(exists $self->{target_snp_hash}{$position2}) {
					$self->{gen_out} = $self->{gen_out} . "{" . $self->{target_snp_hash}{$position2} . "}";
				}
				else{
					$self->{gen_out} .= $value;
				}
			}
			$self->{gen_out} =~ s/<$i>/$primary_snp_pos/;
			$self->{gen_out} .= "\n\n";#"<br><br>";
		}
	}
	# delete dashes outside of snps
	$self->{gen_out} =~ s/([A-Z])-+?([A-Z])/$1$2/g;
	# highlight
	#$self->{gen_out} =~ s/(\[.+?\])/<font color="red"><b>$1<\/b><\/font>/g;
	#$self->{gen_out} =~ s/(\{.+?\})/<font color="blue"><b>$1<\/b><\/font>/g;
}


1;
